// src/runtime/CanvasHooksContext.tsx
import * as React from "react";
var CanvasHooksContext = React.createContext({});

// src/runtime/ToolpadApp.tsx
import * as React13 from "react";
import {
  Stack as Stack2,
  CssBaseline,
  Alert,
  Box as Box4,
  styled,
  AlertTitle,
  LinearProgress,
  Container,
  Tooltip as Tooltip4,
  Typography as Typography2
} from "@mui/material";
import {
  createComponent,
  TOOLPAD_COMPONENT,
  Slots,
  Placeholder,
  getArgTypeDefaultValue,
  ComponentsContextProvider,
  isToolpadComponent,
  createToolpadComponentThatThrows,
  useComponents,
  useComponent
} from "@mui/toolpad-core";
import { createProvidedContext, useAssertedContext } from "@mui/toolpad-utils/react";
import { mapProperties as mapProperties2, mapValues as mapValues3 } from "@mui/toolpad-utils/collections";
import { set as setObjectPath2 } from "lodash-es";
import { QueryClient as QueryClient2, QueryClientProvider, useMutation as useMutation2 } from "@tanstack/react-query";
import {
  BrowserRouter,
  Routes,
  Route,
  useLocation as useLocation2,
  Navigate,
  useNavigate,
  matchPath
} from "react-router-dom";
import { ErrorBoundary } from "react-error-boundary";
import {
  CanvasEventsContext,
  NodeRuntimeWrapper,
  ResetNodeErrorsKeyProvider
} from "@mui/toolpad-core/runtime";
import ErrorIcon from "@mui/icons-material/Error";
import { getBrowserRuntime } from "@mui/toolpad-core/jsBrowserRuntime";
import * as builtIns from "@mui/toolpad-components";
import { errorFrom } from "@mui/toolpad-utils/errors";
import useBoolean from "@mui/toolpad-utils/hooks/useBoolean";
import usePageTitle from "@mui/toolpad-utils/hooks/usePageTitle";
import invariant4 from "invariant";

// src/appDom/index.ts
import { nanoid } from "nanoid/non-secure";
import { generateKeyBetween } from "fractional-indexing";
import invariant from "invariant";
import { pascalCase, removeDiacritics, uncapitalize } from "@mui/toolpad-utils/strings";
import { mapProperties, mapValues, hasOwnProperty } from "@mui/toolpad-utils/collections";

// src/utils/immutability.ts
import { setWith, clone } from "lodash-es";
function updatePath(obj, path, value) {
  return setWith(clone(obj), path, value, clone);
}

// src/server/schema.ts
import { z } from "zod";
function toolpadObjectSchema(kind, spec) {
  return z.object({
    apiVersion: z.literal("v1").describe(
      `Defines the version of this object. Used in determining compatibility between Toolpad "${kind}" objects.`
    ),
    kind: z.literal(kind).describe(`Describes the nature of this Toolpad "${kind}" object.`),
    spec: spec.describe(`Defines the shape of this "${kind}" object`)
  });
}
var literalSchema = z.union([z.string(), z.number(), z.boolean(), z.null()]);
function nameValuePairSchema(valueType) {
  return z.object({
    name: z.string().describe("The name"),
    value: valueType.describe(valueType.description ?? "The value")
  }).describe("A name/value pair.");
}
var jsExpressionBindingSchema = z.object({
  $$jsExpression: z.string().describe("The expression to be evaluated.")
}).describe("A binding that evaluates an expression and returns the result.");
var envBindingSchema = z.object({
  $$env: z.string().describe("The name of an environment variable.")
}).describe("An environment variable.");
function bindableSchema(valueType) {
  return z.union([valueType, jsExpressionBindingSchema, envBindingSchema]);
}
var jsExpressionActionSchema = z.object({
  $$jsExpressionAction: z.string().describe("The code to be executed.")
}).describe("A javascript expression to be executed when this action is triggered.");
var navigationActionSchema = z.object({
  $$navigationAction: z.object({
    page: z.string().describe("The page that is being navigated to"),
    parameters: z.record(bindableSchema(z.any())).describe("Parameters to pass when navigating to this page")
  })
}).describe(
  "A navigation from one page to another, optionally passing parameters to the next page."
);
var fetchModeSchema = z.union([
  z.literal("query").describe("Fetch automatically when the page opens"),
  z.literal("mutation").describe("Fetch on manual action only")
]);
var nameStringValuePairSchema = nameValuePairSchema(z.string()).describe(
  "a name/value pair with a string value."
);
var rawBodySchema = z.object({
  kind: z.literal("raw"),
  content: bindableSchema(z.string()),
  contentType: z.string()
});
var bindableNameStringValueSchema = nameValuePairSchema(bindableSchema(z.string())).describe(
  "A name/value pair where the value is dynamically bindable to strings."
);
var urlEncodedBodySchema = z.object({
  kind: z.literal("urlEncoded"),
  content: z.array(bindableNameStringValueSchema)
});
var fetchBodySchema = z.discriminatedUnion("kind", [rawBodySchema, urlEncodedBodySchema]);
var rawResponseTypeSchema = z.object({
  kind: z.literal("raw")
}).describe("Don't interpret this body at all.");
var jsonResponseTypeSchema = z.object({
  kind: z.literal("json")
}).describe("Interpret the fetch response as JSON");
var csvResponseTypeSchema = z.object({
  kind: z.literal("csv"),
  headers: z.boolean().describe("First row contains headers")
}).describe("Interpret the fetch response as CSV");
var xmlResponseTypeSchema = z.object({
  kind: z.literal("xml")
}).describe("Interpret the fetch response as XML");
var responseTypeSchema = z.discriminatedUnion("kind", [
  rawResponseTypeSchema,
  jsonResponseTypeSchema,
  csvResponseTypeSchema,
  xmlResponseTypeSchema
]).describe("Describes how a the fetch response is to be interpreted.");
var fetchQueryConfigSchema = z.object({
  kind: z.literal("rest").describe("Designates this object as a fetch query."),
  url: bindableSchema(z.string()).optional().describe("The URL of the request"),
  method: z.string().optional().describe("The request method."),
  headers: z.array(bindableNameStringValueSchema).optional().describe("Extra request headers."),
  searchParams: z.array(bindableNameStringValueSchema).optional().describe("Extra url query parameters."),
  body: fetchBodySchema.optional().describe("The request body."),
  transformEnabled: z.boolean().optional().describe("Run a custom transformer on the response."),
  transform: z.string().optional().describe("The custom transformer to run when enabled."),
  response: responseTypeSchema.optional().describe("How to parse the response.")
});
var localQueryConfigSchema = z.object({
  kind: z.literal("local").describe("Designates this object as a local function query."),
  function: z.string().optional().describe("The function to be executed on the backend by this query.")
});
var queryConfigSchema = z.discriminatedUnion("kind", [
  fetchQueryConfigSchema,
  localQueryConfigSchema
]);
var querySchema = z.object({
  name: z.string().describe("A name for the query"),
  enabled: bindableSchema(z.boolean()).optional().describe(
    "Activates or deactivates the query. When deactivated the data won't be loaded when the page opens."
  ),
  parameters: z.array(nameValuePairSchema(bindableSchema(z.any()))).optional().describe("Parameters to pass to this query."),
  mode: fetchModeSchema.optional().describe("How to fetch this query."),
  query: queryConfigSchema.optional().describe("Query definition"),
  transform: z.string().optional().describe("Transformation to run on the response"),
  transformEnabled: z.boolean().optional().describe("Enable the transformation"),
  refetchInterval: z.number().optional().describe("Interval to rerun this query at"),
  cacheTime: z.number().optional().describe("Time to cache before refetching")
});
var elementSchema;
var templateSchema = z.object({
  $$template: z.lazy(() => z.array(elementSchema)).describe("The subtree, that describes the UI to be rendered by the template.")
}).describe("Describes a fragment of Toolpad elements, to be used as a template.");
var baseElementSchema = z.object({
  component: z.string().describe("The component that this element was based on."),
  name: z.string().describe("a name for this component, which is used to reference it inside bindings."),
  layout: z.object({
    horizontalAlign: z.string().optional().describe("Lays out the element along the horizontal axis."),
    verticalAlign: z.string().optional().describe("Lays out the element along the vertical axis."),
    columnSize: z.number().optional().describe("The width this element takes up, expressed in terms of columns on the page.")
  }).optional().describe("Layout properties for this element.")
});
var bindablePropSchema = z.lazy(
  () => z.union([
    ...literalSchema.options,
    z.array(bindablePropSchema),
    z.record(
      z.string().refine((key) => !key.startsWith("$$")),
      bindablePropSchema
    ),
    jsExpressionBindingSchema,
    envBindingSchema,
    jsExpressionActionSchema,
    navigationActionSchema,
    templateSchema
  ])
);
elementSchema = baseElementSchema.extend({
  children: z.lazy(() => z.array(elementSchema).optional()).describe("The children of this element."),
  props: z.lazy(() => z.record(bindablePropSchema).optional()).describe("The properties to configure this instance of the component.")
}).describe("The instance of a component. Used to build user interfaces in pages.");
var pageSchema = toolpadObjectSchema(
  "page",
  z.object({
    id: z.string().describe("Serves as a canonical id of the page."),
    title: z.string().optional().describe("Title for this page."),
    parameters: z.array(nameStringValuePairSchema).optional().describe("Parameters for the page. These can be set inside of the url query string."),
    queries: z.array(querySchema).optional().describe("Queries that are used by the page. These will load data when the page opens."),
    content: z.array(elementSchema).optional().describe("The content of the page. This defines the UI."),
    display: z.union([
      z.literal("standalone").describe("Hide the Toolpad chrome and just display the content of the page"),
      z.literal("shell").describe("Show Toolpad header and navigation.")
    ]).optional().describe(
      "Display mode of the page. This can also be set at runtime with the toolpad-display query parameter"
    )
  })
);
var simplePaletteColorOptionsSchema = z.object({
  main: z.string(),
  light: z.string().optional(),
  dark: z.string().optional(),
  contrastText: z.string().optional()
});
var themeOptionsSchema = z.object({
  // TODO: expand to full MUI theme object
  palette: z.object({
    mode: z.union([z.literal("light"), z.literal("dark")]).optional(),
    primary: simplePaletteColorOptionsSchema.optional(),
    secondary: simplePaletteColorOptionsSchema.optional()
  }).passthrough()
}).passthrough();
var themeSchema = toolpadObjectSchema(
  "theme",
  z.object({
    options: themeOptionsSchema.optional().describe("The ThemeOptions object that gets fed into MUI's createTheme function.")
  })
);

// src/appDom/index.ts
function compareFractionalIndex(index1, index2) {
  if (index1 === index2) {
    return 0;
  }
  return index1 > index2 ? 1 : -1;
}
function isType(node, type) {
  return node.type === type;
}
function assertIsType(node, type) {
  invariant(isType(node, type), `Expected node type "${type}" but got "${node.type}"`);
}
function getMaybeNode(dom, nodeId, type) {
  const node = dom.nodes[nodeId];
  if (!node) {
    return null;
  }
  if (type) {
    assertIsType(node, type);
  }
  return node;
}
function getNode(dom, nodeId, type) {
  const node = getMaybeNode(dom, nodeId, type);
  if (!node) {
    throw new Error(`Node "${nodeId}" not found`);
  }
  return node;
}
function assertIsApp(node) {
  assertIsType(node, "app");
}
function isPage(node) {
  return isType(node, "page");
}
function isElement(node) {
  return isType(node, "element");
}
function assertIsElement(node) {
  assertIsType(node, "element");
}
function isQuery(node) {
  return isType(node, "query");
}
function isMutation(node) {
  return isType(node, "mutation");
}
function getRoot(dom) {
  return getNode(dom, dom.root);
}
function getApp(dom) {
  const app = getRoot(dom);
  assertIsApp(app);
  return app;
}
var childrenMemo = /* @__PURE__ */ new WeakMap();
function getChildNodes(dom, parent) {
  let domChildrenMemo = childrenMemo.get(dom);
  if (!domChildrenMemo) {
    domChildrenMemo = /* @__PURE__ */ new Map();
    childrenMemo.set(dom, domChildrenMemo);
  }
  let result = domChildrenMemo.get(parent.id);
  if (!result) {
    result = {};
    domChildrenMemo.set(parent.id, result);
    const allNodeChildren = Object.values(dom.nodes).filter(
      (node) => node.parentId === parent.id
    );
    for (const child of allNodeChildren) {
      const prop = child.parentProp || "children";
      let existing = result[prop];
      if (!existing) {
        existing = [];
        result[prop] = existing;
      }
      existing.push(child);
    }
    for (const childArray of Object.values(result)) {
      childArray?.sort((node1, node2) => {
        invariant(
          node1.parentIndex && node2.parentIndex,
          `Nodes inside the dom should have a parentIndex if they have a parent`
        );
        return compareFractionalIndex(node1.parentIndex, node2.parentIndex);
      });
    }
  }
  return result;
}
var RENDERTREE_NODES = [
  "app",
  "page",
  "element",
  "query",
  "mutation",
  "theme",
  "codeComponent"
];
var frontendNodes = new Set(RENDERTREE_NODES);

// src/bindings.ts
import { hasOwnProperty as hasOwnProperty2 } from "@mui/toolpad-utils/collections";
function getBindingType(binding) {
  if (binding && typeof binding === "object") {
    if (hasOwnProperty2(binding, "$$jsExpression")) {
      return "jsExpression";
    }
    if (hasOwnProperty2(binding, "$$env")) {
      return "env";
    }
    if (hasOwnProperty2(binding, "$$jsExpressionAction")) {
      return "jsExpressionAction";
    }
    if (hasOwnProperty2(binding, "$$navigationAction")) {
      return "navigationAction";
    }
  }
  return "const";
}
function getBindingValue(binding) {
  if (binding && typeof binding === "object") {
    if (hasOwnProperty2(binding, "$$jsExpression")) {
      return binding.$$jsExpression;
    }
    if (hasOwnProperty2(binding, "$$env")) {
      return binding.$$env;
    }
    if (hasOwnProperty2(binding, "$$jsExpressionAction")) {
      return binding.$$jsExpressionAction;
    }
    if (hasOwnProperty2(binding, "$$navigationAction")) {
      return binding.$$navigationAction;
    }
  }
  return binding;
}

// src/runtime/toolpadComponents/index.tsx
var PAGE_ROW_COMPONENT_ID = "PageRow";
var PAGE_COLUMN_COMPONENT_ID = "PageColumn";
var STACK_COMPONENT_ID = "Stack";
var FORM_COMPONENT_ID = "Form";
var INTERNAL_COMPONENTS = /* @__PURE__ */ new Map([
  [PAGE_ROW_COMPONENT_ID, { displayName: "Row", builtIn: "PageRow", system: true }],
  [PAGE_COLUMN_COMPONENT_ID, { displayName: "Column", builtIn: "PageColumn", system: true }],
  [STACK_COMPONENT_ID, { displayName: "Stack", builtIn: "Stack", system: true }],
  [
    "Autocomplete",
    {
      displayName: "Autocomplete",
      builtIn: "Autocomplete"
    }
  ],
  [
    "Button",
    {
      displayName: "Button",
      builtIn: "Button"
    }
  ],
  ["Image", { displayName: "Image", builtIn: "Image" }],
  ["DataGrid", { displayName: "Data Grid", builtIn: "DataGrid" }],
  ["Chart", { displayName: "Chart", builtIn: "Chart" }],
  ["TextField", { displayName: "Text Field", builtIn: "TextField" }],
  ["DatePicker", { displayName: "Date Picker", builtIn: "DatePicker" }],
  ["FilePicker", { displayName: "File Picker", builtIn: "FilePicker" }],
  ["Text", { displayName: "Text", builtIn: "Text" }],
  ["Select", { displayName: "Select", builtIn: "Select" }],
  ["List", { displayName: "List", builtIn: "List" }],
  ["Paper", { displayName: "Paper", builtIn: "Paper" }],
  ["Tabs", { displayName: "Tabs", builtIn: "Tabs" }],
  ["Container", { displayName: "Container", builtIn: "Container" }],
  ["Metric", { displayName: "Metric", builtIn: "Metric" }],
  [FORM_COMPONENT_ID, { displayName: "Form", builtIn: "Form" }]
]);
function getElementNodeComponentId(elementNode) {
  return elementNode.attributes.component;
}
function isPageRow(elementNode) {
  return getElementNodeComponentId(elementNode) === PAGE_ROW_COMPONENT_ID;
}
function isPageColumn(elementNode) {
  return getElementNodeComponentId(elementNode) === PAGE_COLUMN_COMPONENT_ID;
}
function isPageLayoutComponent(elementNode) {
  return isPageRow(elementNode) || isPageColumn(elementNode);
}

// src/runtime/AppThemeProvider.tsx
import * as React2 from "react";
import { createTheme, ThemeProvider } from "@mui/material";
import { deepmerge } from "@mui/utils";
function createMuiThemeFromToolpadTheme(toolpadTheme = {}) {
  return createTheme(
    deepmerge(
      {
        typography: {
          h1: {
            fontSize: `3.25rem`,
            fontWeight: 800
          },
          h2: {
            fontSize: `2.25rem`,
            fontWeight: 700
          },
          h3: {
            fontSize: `1.75rem`,
            fontWeight: 700
          },
          h4: {
            fontSize: `1.5rem`,
            fontWeight: 700
          },
          h5: {
            fontSize: `1.25rem`,
            fontWeight: 700
          },
          h6: {
            fontSize: `1.15rem`,
            fontWeight: 700
          }
        },
        fontFamilyMonospaced: 'Consolas, Menlo, Monaco, "Andale Mono", "Ubuntu Mono", monospace'
      },
      toolpadTheme
    )
  );
}
function createToolpadAppTheme(dom) {
  const root = getApp(dom);
  const { themes = [] } = getChildNodes(dom, root);
  const themeNode = themes.length > 0 ? themes[0] : null;
  const toolpadTheme = themeNode?.theme;
  return createMuiThemeFromToolpadTheme(toolpadTheme);
}
function AppThemeProvider({ dom, children }) {
  const theme = React2.useMemo(() => createToolpadAppTheme(dom), [dom]);
  return /* @__PURE__ */ React2.createElement(ThemeProvider, { theme }, children);
}

// src/runtime/evalJsBindings.ts
import { set as setObjectPath } from "lodash-es";
import { mapValues as mapValues2 } from "@mui/toolpad-utils/collections";
function flattenDependency(deps, dep, history = /* @__PURE__ */ new Set([dep])) {
  const depDeps = deps.get(dep) ?? /* @__PURE__ */ new Set();
  const result = new Set(depDeps);
  for (const depDep of depDeps) {
    if (!history.has(depDep)) {
      const flat = flattenDependency(deps, depDep, /* @__PURE__ */ new Set([...history, depDep]));
      for (const flatted of flat) {
        result.add(flatted);
      }
    }
  }
  return result;
}
function flattenDependencies(deps) {
  const result = /* @__PURE__ */ new Map();
  for (const dep of deps.keys()) {
    result.set(dep, flattenDependency(deps, dep));
  }
  return result;
}
function bubbleError(flatDependencies, results, bindingId) {
  const result = results[bindingId];
  if (result.error) {
    return result.error;
  }
  const deps = flatDependencies.get(bindingId) ?? /* @__PURE__ */ new Set();
  for (const dep of deps) {
    const depResult = results[dep];
    if (depResult.error) {
      return depResult.error;
    }
  }
  return void 0;
}
function bubbleLoading(flatDependencies, results, bindingId) {
  const result = results[bindingId];
  if (result.loading) {
    return true;
  }
  const deps = flatDependencies.get(bindingId) ?? /* @__PURE__ */ new Set();
  for (const dep of deps) {
    const depResult = results[dep];
    if (depResult.loading) {
      return depResult.loading;
    }
  }
  return false;
}
function buildGlobalScope(base, bindings) {
  const globalScope = { ...base };
  for (const binding of Object.values(bindings)) {
    if (binding.scopePath) {
      const value = binding.result?.value;
      setObjectPath(globalScope, binding.scopePath, value);
    }
  }
  return globalScope;
}
function evalJsBindings(jsRuntime, bindings, globalScope) {
  const bindingsMap = new Map(Object.entries(bindings));
  const bindingIdMap = /* @__PURE__ */ new Map();
  for (const [bindingId, binding] of bindingsMap) {
    if (binding.scopePath) {
      bindingIdMap.set(binding.scopePath, bindingId);
    }
  }
  const computationStatuses = /* @__PURE__ */ new Map();
  let currentParentBinding;
  const dependencies = /* @__PURE__ */ new Map();
  let proxiedScope;
  const evaluateBinding = (bindingId, scopePath) => {
    const binding = bindingId && bindingsMap.get(bindingId);
    if (!binding) {
      return null;
    }
    if (currentParentBinding) {
      let bindingDependencies = dependencies.get(currentParentBinding);
      if (!bindingDependencies) {
        bindingDependencies = /* @__PURE__ */ new Set();
        dependencies.set(currentParentBinding, bindingDependencies);
      }
      bindingDependencies.add(bindingId);
    }
    const expression = binding.expression;
    if (expression) {
      const computed = computationStatuses.get(expression);
      if (computed) {
        if (computed.result) {
          return computed.result;
        }
        throw new Error(`Cycle detected "${scopePath}"`);
      }
      computationStatuses.set(expression, { result: null });
      const prevContext = currentParentBinding;
      currentParentBinding = bindingId;
      const result = jsRuntime.evaluateExpression(expression, proxiedScope);
      currentParentBinding = prevContext;
      computationStatuses.set(expression, { result });
      return result;
    }
    if (binding.result) {
      if (binding.dependencies) {
        dependencies.set(bindingId, new Set(binding.dependencies));
      }
      return binding.result;
    }
    const initializer = binding.initializer;
    if (initializer) {
      const result = evaluateBinding(initializer, scopePath);
      if (result) {
        return result;
      }
    }
    return null;
  };
  const proxify = (obj, label) => new Proxy(obj, {
    get(target, prop, receiver) {
      if (typeof prop === "symbol") {
        return Reflect.get(target, prop, receiver);
      }
      const scopePath = label ? `${label}.${prop}` : prop;
      const bindingId = bindingIdMap.get(scopePath);
      if (bindingId) {
        const evaluated = evaluateBinding(bindingId, scopePath);
        if (evaluated) {
          return evaluated.value;
        }
      }
      const result = target[prop];
      if (result && typeof result === "object") {
        return proxify(result, scopePath);
      }
      return Reflect.get(target, prop, receiver);
    }
  });
  const scope = buildGlobalScope(globalScope, bindings);
  proxiedScope = proxify(scope);
  const results = mapValues2(bindings, (binding, bindingId) => {
    return evaluateBinding(bindingId) || { value: void 0 };
  });
  const flatDependencies = flattenDependencies(dependencies);
  return mapValues2(bindings, (binding, bindingId) => {
    const { scopePath } = binding;
    let bindingResult = results[bindingId];
    let nestedBindingsLoading;
    let nestedBindingsError;
    const mergeNestedBindings = (value, parentBindingId) => {
      if (value && typeof value === "object") {
        for (const nestedPropName of Object.keys(value)) {
          const nestedBindingId = `${parentBindingId}${Array.isArray(value) ? `[${nestedPropName}]` : `.${nestedPropName}`}`;
          const nestedBindingResult = results[nestedBindingId];
          if (nestedBindingResult) {
            if (!nestedBindingsError) {
              nestedBindingsError = bubbleError(flatDependencies, results, nestedBindingId);
            }
            if (!nestedBindingsLoading) {
              nestedBindingsLoading = bubbleLoading(flatDependencies, results, nestedBindingId);
            }
            bindingResult = updatePath(
              bindingResult,
              `value.${nestedBindingId.replace(bindingId, "")}`,
              nestedBindingResult.value
            );
          } else {
            mergeNestedBindings(
              value[nestedPropName],
              nestedBindingId
            );
          }
        }
      }
    };
    mergeNestedBindings(bindingResult.value, bindingId);
    return {
      scopePath,
      dependencies: Array.from(flatDependencies.get(bindingId) ?? []),
      result: {
        ...bindingResult,
        error: nestedBindingsError || bubbleError(flatDependencies, results, bindingId),
        loading: nestedBindingsLoading || bubbleLoading(flatDependencies, results, bindingId)
      }
    };
  });
}

// src/runtime/constants.ts
var HTML_ID_EDITOR_OVERLAY = "editor-overlay";

// src/runtime/toolpadComponents/layoutBox.ts
var layoutBoxArgTypes = {
  horizontalAlign: {
    helperText: "Horizontal alignment of the component",
    type: "string",
    enum: ["start", "center", "end", "space-between", "space-around", "space-evenly"],
    default: "start",
    label: "Horizontal alignment",
    control: { type: "HorizontalAlign", hideLabel: true }
  },
  verticalAlign: {
    helperText: "Vertical alignment of the component.",
    type: "string",
    enum: ["start", "center", "end", "space-between", "space-around", "space-evenly"],
    default: "start",
    label: "Vertical alignment",
    control: { type: "VerticalAlign", hideLabel: true }
  }
};

// src/runtime/useDataQuery.ts
import * as React3 from "react";
import { useQuery } from "@tanstack/react-query";
async function execDataSourceQuery({
  signal,
  pageName,
  queryName,
  params
}) {
  const dataUrl = new URL(`${process.env.BASE_URL}/api/data/`, window.location.href);
  const url = new URL(
    `./${encodeURIComponent(pageName)}/${encodeURIComponent(queryName)}`,
    dataUrl
  );
  const res = await fetch(String(url), {
    method: "POST",
    body: JSON.stringify(params),
    headers: [["content-type", "application/json"]],
    signal
  });
  if (!res.ok) {
    throw new Error(`HTTP ${res.status} while fetching "${url}"`);
  }
  return res.json();
}
var EMPTY_ARRAY = [];
var EMPTY_OBJECT = {};
function useDataQuery(page, node, params, {
  enabled = true,
  ...options2
}) {
  const { savedNodes } = React3.useContext(CanvasHooksContext);
  const queryName = node.name;
  const pageName = page.name;
  const nodeHash = savedNodes ? savedNodes[node.id] : void 0;
  const isNodeAvailableOnServer = savedNodes ? !!savedNodes[node.id] : true;
  const {
    isLoading,
    isFetching,
    error: fetchError,
    data: responseData = EMPTY_OBJECT,
    refetch
  } = useQuery(
    [nodeHash, pageName, queryName, params],
    ({ signal }) => execDataSourceQuery({ signal, pageName, queryName, params }),
    {
      ...options2,
      enabled: isNodeAvailableOnServer && enabled
    }
  );
  const { data, error: apiError } = responseData;
  const error = apiError || fetchError;
  const rows = Array.isArray(data) ? data : EMPTY_ARRAY;
  const result = React3.useMemo(
    () => ({
      isLoading: isLoading && enabled,
      isFetching,
      error,
      data,
      rows,
      refetch,
      fetch: async () => {
        throw new Error(`"fetch" is unsupported for automatic queries`);
      },
      call: async () => {
        throw new Error(`"call" is unsupported for automatic queries`);
      }
    }),
    [isLoading, enabled, isFetching, error, data, rows, refetch]
  );
  return result;
}

// src/runtime/AppNavigation.tsx
import * as React4 from "react";
import {
  Drawer,
  Box,
  List,
  ListSubheader,
  ListItem,
  ListItemButton,
  ListItemText,
  Toolbar
} from "@mui/material";
import { useLocation, useHref, Link } from "react-router-dom";
var DRAWER_WIDTH = 250;
function AppNavigation({ pages, clipped = false }) {
  const location = useLocation();
  const { search } = location;
  const href = useHref("");
  const activePagePath = location.pathname.replace(href, "");
  const navListSubheaderId = React4.useId();
  return /* @__PURE__ */ React4.createElement(
    Drawer,
    {
      variant: "permanent",
      anchor: "left",
      open: true,
      sx: {
        width: DRAWER_WIDTH,
        flexShrink: 0,
        [`& .MuiDrawer-paper`]: { width: DRAWER_WIDTH, boxSizing: "border-box" }
      }
    },
    clipped ? /* @__PURE__ */ React4.createElement(Toolbar, { variant: "dense" }) : null,
    /* @__PURE__ */ React4.createElement(Box, null, /* @__PURE__ */ React4.createElement(
      List,
      {
        component: "nav",
        subheader: /* @__PURE__ */ React4.createElement(ListSubheader, { id: navListSubheaderId, sx: { px: 4 } }, "Pages"),
        "aria-labelledby": navListSubheaderId
      },
      pages.map((page) => /* @__PURE__ */ React4.createElement(ListItem, { key: page.id, disablePadding: true }, /* @__PURE__ */ React4.createElement(
        ListItemButton,
        {
          component: Link,
          to: `pages/${page.id}${search}`,
          selected: activePagePath === `/pages/${page.id}`
        },
        /* @__PURE__ */ React4.createElement(ListItemText, { primary: page.name, sx: { ml: 2 } })
      )))
    ))
  );
}

// src/runtime/PreviewHeader.tsx
import * as React11 from "react";
import { Stack, Button as Button2, Typography } from "@mui/material";
import EditIcon from "@mui/icons-material/Edit";

// src/toolpad/ToolpadShell/Header/index.tsx
import * as React10 from "react";
import { AppBar, Box as Box3, Toolbar as Toolbar2, Tooltip as Tooltip3, Chip as Chip2, Link as Link2, useTheme } from "@mui/material";

// src/toolpad/ToolpadShell/Header/UserFeedback/index.tsx
import * as React6 from "react";
import {
  Chip,
  Divider,
  ListItemText as ListItemText2,
  IconButton,
  Menu,
  MenuItem,
  Tooltip,
  Button,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  Box as Box2
} from "@mui/material";
import HelpOutlinedIcon from "@mui/icons-material/HelpOutlined";
import OpenInNewIcon from "@mui/icons-material/OpenInNew";
import invariant3 from "invariant";

// src/utils/useMenu.ts
import * as React5 from "react";
function useMenu() {
  const buttonId = React5.useId();
  const menuId = React5.useId();
  const [anchorEl, setAnchorEl] = React5.useState(null);
  const open = Boolean(anchorEl);
  const onMenuClose = React5.useCallback((event) => {
    event?.stopPropagation?.();
    setAnchorEl(null);
  }, []);
  const buttonProps = React5.useMemo(
    () => ({
      id: buttonId,
      "aria-controls": open ? menuId : void 0,
      "aria-haspopup": "true",
      "aria-expanded": open ? "true" : void 0,
      onClick(event) {
        event.stopPropagation();
        setAnchorEl(event.currentTarget);
      }
    }),
    [buttonId, menuId, open]
  );
  const menuProps = React5.useMemo(
    () => ({
      id: menuId,
      anchorEl,
      open,
      onClose: onMenuClose,
      MenuListProps: {
        "aria-labelledby": buttonId
      }
    }),
    [anchorEl, buttonId, menuId, onMenuClose, open]
  );
  return {
    buttonProps,
    menuProps,
    onMenuClose
  };
}

// src/constants.ts
var TOOLPAD_TARGET_CE = "CE";
var TOOLPAD_TARGET_CLOUD = "CLOUD";
var TOOLPAD_TARGET_PRO = "PRO";
var DOCUMENTATION_URL = "https://mui.com/toolpad/getting-started/overview/";
var TOOLPAD_BRIDGE_GLOBAL = "__TOOLPAD_BRIDGE__";
var VERSION_CHECK_INTERVAL = 1e3 * 60 * 10;

// src/api.ts
import invariant2 from "invariant";
import {
  QueryClient,
  useMutation,
  useQuery as useQuery2
} from "@tanstack/react-query";

// src/rpcClient.ts
import { parse as superjsonParse } from "superjson";
function createFetcher(endpoint, type) {
  return new Proxy(
    {},
    {
      get(target, prop) {
        return async (...params) => {
          const body = {
            type,
            name: prop,
            params
          };
          const res = await fetch(endpoint, {
            method: "POST",
            headers: {
              "content-type": "application/json"
            },
            body: JSON.stringify(body)
          });
          if (res.ok) {
            const response = await res.json();
            if (response.error) {
              const toolpadError = new Error(response.error.message, {
                cause: response.error
              });
              if (response.error.code) {
                toolpadError.code = response.error.code;
              }
              throw toolpadError;
            }
            return superjsonParse(response.result);
          }
          throw new Error(`HTTP ${res.status}`);
        };
      }
    }
  );
}
function createRpcClient(endpoint) {
  const query = createFetcher(endpoint, "query");
  const mutation = createFetcher(endpoint, "mutation");
  return { query, mutation };
}

// src/api.ts
var queryClient = new QueryClient({
  defaultOptions: {
    queries: {
      networkMode: "always"
    },
    mutations: {
      networkMode: "always"
    }
  }
});
function createClient(endpoint) {
  const { query, mutation } = createRpcClient(endpoint);
  return {
    query,
    mutation,
    useQuery: (key, params, options2) => {
      return useQuery2({
        ...options2,
        enabled: !!params && options2?.enabled !== false,
        queryKey: [key, params],
        queryFn: () => {
          invariant2(params, `"enabled" prop of useQuery should prevent this call'`);
          return query[key](...params);
        }
      });
    },
    useMutation: (key, options2) => useMutation((params) => mutation[key](...params), options2),
    refetchQueries(key, params) {
      return queryClient.refetchQueries(params ? [key, params] : [key]);
    },
    invalidateQueries(key, params) {
      return queryClient.invalidateQueries(params ? [key, params] : [key]);
    }
  };
}
var api_default = createClient("/api/rpc");

// src/utils/useBoolean.ts
import { default as default2 } from "@mui/toolpad-utils/hooks/useBoolean";

// src/toolpad/ToolpadShell/Header/UserFeedback/index.tsx
var REPORT_BUG_URL = "https://github.com/mui/mui-toolpad/issues/new?assignees=&labels=status%3A+needs+triage&template=1.bug.yml";
var FEATURE_REQUEST_URL = "https://github.com/mui/mui-toolpad/issues";
function CliCommandSnippet({ children }) {
  return /* @__PURE__ */ React6.createElement(
    Box2,
    {
      component: "pre",
      sx: {
        p: 1,
        backgroundColor: "background.paper",
        borderRadius: 1,
        border: 1,
        borderColor: "divider"
      }
    },
    children
  );
}
function FeedbackMenuItemLink({ href, children }) {
  return /* @__PURE__ */ React6.createElement(MenuItem, { component: "a", target: "_blank", href }, /* @__PURE__ */ React6.createElement(ListItemText2, null, children), /* @__PURE__ */ React6.createElement(OpenInNewIcon, { fontSize: "inherit", sx: { ml: 3, color: "text.secondary" } }));
}
function getReadableTarget() {
  switch (process.env.TOOLPAD_TARGET) {
    case TOOLPAD_TARGET_CLOUD:
      return "Cloud";
    case TOOLPAD_TARGET_CE:
      return "Community Edition";
    case TOOLPAD_TARGET_PRO:
      return "Pro";
    default:
      return "Unknown";
  }
}
function getUpgradeMessage(packageManager) {
  const pkgName = "@mui/toolpad";
  switch (packageManager) {
    case "yarn":
      return `yarn add ${pkgName}`;
    case "pnpm":
      return `pnpm add ${pkgName}`;
    default:
      return `npm install ${pkgName}`;
  }
}
function UserFeedback() {
  const { buttonProps, menuProps } = useMenu();
  invariant3(process.env.TOOLPAD_VERSION, "Missing env var TOOLPAD_VERSION");
  invariant3(process.env.TOOLPAD_BUILD, "Missing env var TOOLPAD_BUILD");
  const { data: versionInfo } = api_default.useQuery("getVersionInfo", [], {
    staleTime: VERSION_CHECK_INTERVAL
  });
  const {
    value: updateDialogOpen,
    setFalse: handleUpdateDialogClose,
    setTrue: handleUpdateDialogOpen
  } = default2(false);
  const updateAvailable = !!versionInfo?.updateAvailable;
  return /* @__PURE__ */ React6.createElement(React6.Fragment, null, /* @__PURE__ */ React6.createElement(Dialog, { open: updateDialogOpen, onClose: handleUpdateDialogClose, maxWidth: "xs" }, /* @__PURE__ */ React6.createElement(DialogTitle, null, "Update Toolpad"), /* @__PURE__ */ React6.createElement(DialogContent, null, /* @__PURE__ */ React6.createElement(DialogContentText, null, "A new Toolpad version is available. To upgrade to the latest version, run:", /* @__PURE__ */ React6.createElement(CliCommandSnippet, null, getUpgradeMessage(versionInfo?.packageManager ?? null)))), /* @__PURE__ */ React6.createElement(DialogActions, null, /* @__PURE__ */ React6.createElement(Button, { onClick: handleUpdateDialogClose, autoFocus: true }, "Close"))), /* @__PURE__ */ React6.createElement(Tooltip, { title: "Help and resources" }, /* @__PURE__ */ React6.createElement(IconButton, { ...buttonProps, color: "primary" }, /* @__PURE__ */ React6.createElement(HelpOutlinedIcon, null))), /* @__PURE__ */ React6.createElement(Menu, { ...menuProps }, /* @__PURE__ */ React6.createElement(FeedbackMenuItemLink, { href: DOCUMENTATION_URL }, "Documentation"), /* @__PURE__ */ React6.createElement(FeedbackMenuItemLink, { href: REPORT_BUG_URL }, "Report bug"), /* @__PURE__ */ React6.createElement(FeedbackMenuItemLink, { href: FEATURE_REQUEST_URL }, "Request or upvote feature"), /* @__PURE__ */ React6.createElement(Divider, null), /* @__PURE__ */ React6.createElement(MenuItem, { disabled: true }, getReadableTarget()), /* @__PURE__ */ React6.createElement(
    MenuItem,
    {
      disabled: !updateAvailable,
      onClick: handleUpdateDialogOpen,
      sx: { justifyContent: "space-between" }
    },
    "Version ",
    process.env.TOOLPAD_VERSION,
    updateAvailable ? /* @__PURE__ */ React6.createElement(Chip, { size: "small", color: "error", variant: "outlined", label: "Update", clickable: true }) : null
  ), /* @__PURE__ */ React6.createElement(MenuItem, { disabled: true }, "Build ", process.env.TOOLPAD_BUILD)));
}
var UserFeedback_default = UserFeedback;

// src/toolpad/ToolpadShell/Header/ThemeModeMenu.tsx
import * as React7 from "react";
import { IconButton as IconButton2, Menu as Menu2, MenuItem as MenuItem2, ListItemText as ListItemText3, ListItemIcon, Tooltip as Tooltip2 } from "@mui/material";
import LightModeOutlined from "@mui/icons-material/LightMode";
import DarkModeOutlined from "@mui/icons-material/DarkMode";
import SettingsBrightnessOutlined from "@mui/icons-material/SettingsBrightnessOutlined";
var options = {
  light: {
    label: "Light",
    value: "light",
    icon: /* @__PURE__ */ React7.createElement(LightModeOutlined, null)
  },
  dark: {
    label: "Dark",
    value: "dark",
    icon: /* @__PURE__ */ React7.createElement(DarkModeOutlined, null)
  },
  system: {
    label: "System",
    value: "system",
    icon: /* @__PURE__ */ React7.createElement(SettingsBrightnessOutlined, null)
  }
};
function ThemeModeMenu({ mode, onChange }) {
  const { buttonProps, menuProps, onMenuClose } = useMenu();
  return /* @__PURE__ */ React7.createElement(React7.Fragment, null, /* @__PURE__ */ React7.createElement(Tooltip2, { title: "Change theme" }, /* @__PURE__ */ React7.createElement(IconButton2, { ...buttonProps, "aria-label": "Change theme", color: "primary" }, options[mode].icon)), /* @__PURE__ */ React7.createElement(Menu2, { ...menuProps }, Object.values(options).map((option) => /* @__PURE__ */ React7.createElement(
    MenuItem2,
    {
      key: option.value,
      selected: option.value === mode,
      onClick: (event) => {
        onChange(event, option.value);
        onMenuClose();
      }
    },
    /* @__PURE__ */ React7.createElement(ListItemIcon, null, option.icon),
    /* @__PURE__ */ React7.createElement(ListItemText3, { primary: option.label })
  ))));
}
var ThemeModeMenu_default = ThemeModeMenu;

// src/ThemeContext.tsx
import * as React9 from "react";
import { ScopedCssBaseline } from "@mui/material";
import { ThemeProvider as MuiThemeProvider, createTheme as createTheme3 } from "@mui/material/styles";
import { deepmerge as deepmerge2 } from "@mui/utils";
import useMediaQuery from "@mui/material/useMediaQuery";

// src/theme.ts
import ArrowDropDownRounded from "@mui/icons-material/ArrowDropDownRounded";
import { tooltipClasses } from "@mui/material";
import { createTheme as createTheme2, alpha } from "@mui/material/styles";
var defaultTheme = createTheme2();
var blue = {
  50: "#F0F7FF",
  100: "#C2E0FF",
  200: "#99CCF3",
  300: "#66B2FF",
  400: "#3399FF",
  main: "#007FFF",
  500: "#007FFF",
  600: "#0072E5",
  // vs blueDark 900: WCAG 4.6 AAA (large), APCA 36 Not for reading text
  700: "#0059B2",
  800: "#004C99",
  900: "#003A75"
};
var blueDark = {
  50: "#E2EDF8",
  100: "#CEE0F3",
  200: "#91B9E3",
  300: "#5090D3",
  main: "#5090D3",
  400: "#265D97",
  500: "#1E4976",
  600: "#173A5E",
  700: "#132F4C",
  // contrast 13.64:1
  800: "#001E3C",
  900: "#0A1929"
};
var grey = {
  50: "#F3F6F9",
  100: "#E7EBF0",
  200: "#E0E3E7",
  300: "#CDD2D7",
  // vs blueDark 900: WCAG 11.6 AAA, APCA 78 Best for text
  400: "#B2BAC2",
  // vs blueDark 900: WCAG 9 AAA, APCA 63.3 Ok for text
  500: "#A0AAB4",
  // vs blueDark 900: WCAG 7.5 AAA, APCA 54.3 Only for large text
  600: "#6F7E8C",
  // vs white bg: WCAG 4.1 AA, APCA 68.7 Ok for text
  700: "#3E5060",
  // vs white bg: WCAG 8.3 AAA, APCA 88.7 Best for text
  800: "#2D3843",
  // vs white bg: WCAG 11.9 AAA, APCA 97.3 Best for text
  900: "#1A2027"
};
var systemFont = [
  "-apple-system",
  "BlinkMacSystemFont",
  '"Segoe UI"',
  "Roboto",
  '"Helvetica Neue"',
  "Arial",
  "sans-serif",
  '"Apple Color Emoji"',
  '"Segoe UI Emoji"',
  '"Segoe UI Symbol"'
];
var getMetaThemeColor = (mode) => {
  const themeColor = {
    light: grey[50],
    dark: blueDark[800]
  };
  return themeColor[mode];
};
var getDesignTokens = (mode) => ({
  palette: {
    primary: {
      ...blue,
      ...mode === "dark" && {
        main: blue[400]
      }
    },
    divider: mode === "dark" ? alpha(blue[100], 0.08) : grey[100],
    primaryDark: blueDark,
    mode,
    ...mode === "dark" && {
      background: {
        default: blueDark[800],
        paper: blueDark[900]
      }
    },
    common: {
      black: "#1D1D1D"
    },
    text: {
      ...mode === "light" && {
        primary: grey[900],
        secondary: grey[700]
      },
      ...mode === "dark" && {
        primary: "#fff",
        secondary: grey[400]
      }
    },
    grey: {
      ...mode === "light" && {
        main: grey[100],
        contrastText: grey[600]
      },
      ...mode === "dark" && {
        main: blueDark[700],
        contrastText: grey[600]
      }
    },
    error: {
      50: "#FFF0F1",
      100: "#FFDBDE",
      200: "#FFBDC2",
      300: "#FF99A2",
      400: "#FF7A86",
      500: "#FF505F",
      main: "#EB0014",
      // contrast 4.63:1
      600: "#EB0014",
      700: "#C70011",
      800: "#94000D",
      900: "#570007"
    },
    success: {
      50: "#E9FBF0",
      100: "#C6F6D9",
      200: "#9AEFBC",
      300: "#6AE79C",
      400: "#3EE07F",
      500: "#21CC66",
      600: "#1DB45A",
      ...mode === "light" && {
        main: "#1AA251"
        // contrast 3.31:1
      },
      ...mode === "dark" && {
        main: "#1DB45A"
        // contrast 6.17:1 (blueDark.800)
      },
      700: "#1AA251",
      800: "#178D46",
      900: "#0F5C2E"
    },
    warning: {
      50: "#FFF9EB",
      100: "#FFF3C1",
      200: "#FFECA1",
      300: "#FFDC48",
      // vs blueDark900: WCAG 10.4 AAA, APCA 72 Ok for text
      400: "#F4C000",
      // vs blueDark900: WCAG 6.4 AA normal, APCA 48 Only large text
      500: "#DEA500",
      // vs blueDark900: WCAG 8 AAA normal, APCA 58 Only large text
      main: "#DEA500",
      600: "#D18E00",
      // vs blueDark900: WCAG 6.4 AA normal, APCA 48 Only large text
      700: "#AB6800",
      // vs white bg: WCAG 4.4 AA large, APCA 71 Ok for text
      800: "#8C5800",
      // vs white bg: WCAG 5.9 AAA large, APCA 80 Best for text
      900: "#5A3600"
      // vs white bg: WCAG 10.7 AAA, APCA 95 Best for text
    }
  },
  shape: {
    borderRadius: 10
  },
  spacing: 10,
  typography: {
    fontFamily: ['"IBM Plex Sans"', ...systemFont].join(","),
    fontFamilyCode: [
      "Consolas",
      "Menlo",
      "Monaco",
      "Andale Mono",
      "Ubuntu Mono",
      "monospace"
    ].join(","),
    fontFamilyTagline: ['"PlusJakartaSans-ExtraBold"', ...systemFont].join(","),
    fontFamilySystem: systemFont.join(","),
    fontWeightSemiBold: 600,
    fontWeightExtraBold: 800,
    h1: {
      fontFamily: ['"PlusJakartaSans-ExtraBold"', ...systemFont].join(","),
      fontSize: "clamp(2.625rem, 1.2857rem + 3.5714vw, 4rem)",
      fontWeight: 800,
      lineHeight: 78 / 70,
      ...mode === "light" && {
        color: blueDark[900]
      }
    },
    h2: {
      fontFamily: ['"PlusJakartaSans-ExtraBold"', ...systemFont].join(","),
      fontSize: "clamp(1.5rem, 0.9643rem + 1.4286vw, 2.25rem)",
      fontWeight: 800,
      lineHeight: 44 / 36,
      color: mode === "dark" ? grey[100] : blueDark[700]
    },
    h3: {
      fontFamily: ['"PlusJakartaSans-Bold"', ...systemFont].join(","),
      fontSize: defaultTheme.typography.pxToRem(36),
      lineHeight: 44 / 36,
      letterSpacing: 0.2
    },
    h4: {
      fontFamily: ['"PlusJakartaSans-Bold"', ...systemFont].join(","),
      fontSize: defaultTheme.typography.pxToRem(28),
      lineHeight: 42 / 28,
      letterSpacing: 0.2
    },
    h5: {
      fontFamily: ['"PlusJakartaSans-Bold"', ...systemFont].join(","),
      fontSize: defaultTheme.typography.pxToRem(24),
      lineHeight: 36 / 24,
      letterSpacing: 0.1,
      color: mode === "dark" ? blue[300] : blue.main
    },
    h6: {
      fontSize: defaultTheme.typography.pxToRem(20),
      lineHeight: 30 / 20
    },
    button: {
      textTransform: "initial",
      fontWeight: 700,
      letterSpacing: 0
    },
    subtitle1: {
      fontSize: defaultTheme.typography.pxToRem(18),
      lineHeight: 24 / 18,
      letterSpacing: 0,
      fontWeight: 500
    },
    body1: {
      fontSize: defaultTheme.typography.pxToRem(16),
      // 16px
      lineHeight: 24 / 16,
      letterSpacing: 0
    },
    body2: {
      fontSize: defaultTheme.typography.pxToRem(14),
      // 14px
      lineHeight: 21 / 14,
      letterSpacing: 0
    },
    caption: {
      fontSize: defaultTheme.typography.pxToRem(12),
      // 12px
      lineHeight: 18 / 12,
      letterSpacing: 0,
      fontWeight: 700
    },
    allVariants: {
      scrollMarginTop: "calc(var(--MuiDocs-header-height) + 32px)"
    }
  }
});
function getThemedComponents(theme) {
  return {
    components: {
      MuiButtonBase: {
        defaultProps: {
          disableTouchRipple: true
        }
      },
      MuiButton: {
        defaultProps: {
          disableElevation: true,
          size: "small"
        },
        styleOverrides: {
          sizeLarge: {
            padding: "0.875rem 1rem",
            ...theme.typography.body1,
            lineHeight: 21 / 16,
            fontWeight: 700
          },
          sizeSmall: {
            padding: theme.spacing(0.5, 1)
          },
          containedPrimary: {
            backgroundColor: theme.palette.primary[500],
            color: "#fff"
          }
        },
        variants: [
          {
            // @ts-ignore internal repo module augmentation issue
            props: { variant: "code" },
            style: {
              color: theme.palette.mode === "dark" ? theme.palette.grey[400] : theme.palette.grey[800],
              border: "1px solid",
              borderColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[400] : theme.palette.grey[300],
              backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.grey[50],
              fontFamily: theme.typography.fontFamilyCode,
              fontWeight: 400,
              fontSize: defaultTheme.typography.pxToRem(13),
              // 14px
              lineHeight: 21 / 14,
              letterSpacing: 0,
              WebkitFontSmoothing: "subpixel-antialiased",
              "&:hover, &.Mui-focusVisible": {
                borderColor: theme.palette.primary.main,
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.primary[50],
                "& .MuiButton-endIcon": {
                  color: theme.palette.mode === "dark" ? theme.palette.primary[300] : theme.palette.primary.main
                }
              },
              "& .MuiButton-startIcon": {
                color: theme.palette.grey[400]
              },
              "& .MuiButton-endIcon": {
                display: "inline-block",
                position: "absolute",
                right: 0,
                marginRight: 10,
                color: theme.palette.mode === "dark" ? theme.palette.grey[400] : theme.palette.grey[700]
              }
            }
          },
          {
            // @ts-ignore internal repo module augmentation issue
            props: { variant: "link" },
            style: {
              fontSize: theme.typography.pxToRem(14),
              fontWeight: 700,
              color: theme.palette.mode === "dark" ? theme.palette.primary[300] : theme.palette.primary[600],
              mb: 1,
              "& svg": {
                ml: -0.5
              }
            }
          }
        ]
      },
      MuiIconButton: {
        defaultProps: {
          size: "small"
        },
        variants: [
          {
            props: { color: "primary" },
            style: {
              height: 34,
              width: 34,
              border: `1px solid ${theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.grey[200]}`,
              borderRadius: theme.shape.borderRadius,
              color: theme.palette.mode === "dark" ? theme.palette.primary[300] : theme.palette.primary[500],
              "&:hover": {
                borderColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.grey[300],
                background: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[700], 0.4) : theme.palette.grey[50]
              }
            }
          }
        ]
      },
      MuiMenu: {
        styleOverrides: {
          paper: {
            minWidth: 160,
            color: theme.palette.text.secondary,
            backgroundImage: "none",
            backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[900] : theme.palette.background.paper,
            border: `1px solid ${theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.grey[200]}`,
            "& .MuiMenuItem-root": {
              fontSize: theme.typography.pxToRem(14),
              fontWeight: 500,
              "&:hover": {
                backgroundColor: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[700], 0.4) : theme.palette.grey[50]
              },
              "&:focus": {
                backgroundColor: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[700], 0.4) : theme.palette.grey[50]
              },
              "&.Mui-selected": {
                fontWeight: 500,
                color: theme.palette.mode === "dark" ? theme.palette.primary[300] : theme.palette.primary[600],
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : alpha(theme.palette.primary[100], 0.6)
              }
            }
          }
        }
      },
      MuiPopover: {
        styleOverrides: {
          paper: {
            boxShadow: `0px 4px 20px ${theme.palette.mode === "dark" ? "rgba(0, 0, 0, 0.5)" : "rgba(170, 180, 190, 0.3)"}`
          }
        }
      },
      MuiDivider: {
        styleOverrides: {
          root: {
            borderColor: theme.palette.mode === "dark" ? alpha(theme.palette.primary[100], 0.08) : theme.palette.grey[100]
          }
        }
      },
      MuiLink: {
        defaultProps: {
          underline: "none"
        },
        styleOverrides: {
          root: {
            color: theme.palette.mode === "dark" ? theme.palette.primary[300] : theme.palette.primary[600],
            fontWeight: 700,
            alignItems: "center",
            "&:hover": {
              color: theme.palette.mode === "dark" ? theme.palette.primary[200] : theme.palette.primary[700]
            },
            "&.MuiTypography-body1 > svg": {
              marginTop: 2
            },
            "& svg:last-child": {
              marginLeft: 2
            },
            [`.${tooltipClasses.tooltip} &`]: {
              // Tooltip has a dark background in light mode
              color: theme.palette.primary[300],
              "&:hover": {
                color: theme.palette.primary[200]
              }
            }
          }
        }
      },
      MuiChip: {
        styleOverrides: {
          root: ({ ownerState: { color, variant } }) => ({
            fontWeight: 500,
            ...variant === "outlined" && color === "default" && {
              color: theme.palette.mode === "dark" ? theme.palette.grey[300] : theme.palette.grey[900],
              backgroundColor: "transparent",
              borderColor: theme.palette.mode === "dark" ? alpha(theme.palette.grey[100], 0.1) : theme.palette.grey[200],
              "&:hover": {
                color: theme.palette.mode === "dark" ? theme.palette.grey[300] : theme.palette.grey[900]
              }
            },
            ...variant === "outlined" && color === "primary" && {
              "&:hover": {
                color: theme.palette.primary[500]
              }
            },
            ...variant === "filled" && color === "default" && {
              border: "1px solid transparent",
              color: theme.palette.mode === "dark" ? "#fff" : theme.palette.primary[700],
              backgroundColor: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[500], 0.8) : alpha(theme.palette.primary[100], 0.5),
              "&:hover": {
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.primary[100]
              }
            },
            // for labelling product in the search
            // @ts-ignore internal repo module augmentation issue
            ...variant === "light" && {
              ...color === "default" && {
                color: theme.palette.mode === "dark" ? theme.palette.primary[200] : theme.palette.primary[700],
                backgroundColor: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[700], 0.5) : alpha(theme.palette.primary[100], 0.3)
              },
              ...color === "warning" && {
                color: theme.palette.mode === "dark" ? "#fff" : theme.palette.warning[900],
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.warning[900] : theme.palette.warning[100]
              },
              ...color === "success" && {
                color: theme.palette.mode === "dark" ? "#fff" : theme.palette.success[900],
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.success[900] : theme.palette.success[100]
              }
            }
          }),
          deleteIcon: {
            color: theme.palette.mode === "dark" ? "#fff" : theme.palette.primary[700],
            "&:hover": {
              color: theme.palette.mode === "dark" ? theme.palette.grey[100] : theme.palette.primary[900]
            }
          }
        }
      },
      MuiList: {
        defaultProps: {
          dense: true
        },
        styleOverrides: {
          root: {
            padding: 0
          }
        }
      },
      MuiListItemButton: {
        defaultProps: {
          dense: true
        },
        styleOverrides: {
          root: {
            padding: "8px",
            textTransform: "none",
            fontWeight: 500,
            fontSize: theme.typography.pxToRem(14),
            color: theme.palette.mode === "dark" ? theme.palette.grey[300] : theme.palette.grey[700],
            borderRadius: 0,
            "&:hover": {
              backgroundColor: theme.palette.mode === "dark" ? alpha(theme.palette.primaryDark[700], 0.4) : theme.palette.grey[50]
            },
            "&.Mui-selected": {
              color: theme.palette.mode === "dark" ? "#fff" : theme.palette.primary[500],
              borderRadius: 10,
              border: "1px solid",
              borderColor: theme.palette.mode === "dark" ? `${theme.palette.primary[700]} !important` : `${theme.palette.primary[500]} !important`,
              backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.primary[50],
              "&:hover": {
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.primary[100]
              }
            },
            "&.MuiListItemButton-dense": {
              padding: "4px 16px"
            }
          }
        }
      },
      MuiSelect: {
        defaultProps: {
          IconComponent: ArrowDropDownRounded,
          size: "small"
        },
        styleOverrides: {
          iconFilled: {
            top: "calc(50% - .25em)"
          }
        }
      },
      MuiTab: {
        styleOverrides: {
          root: {
            padding: 8,
            minHeight: 0
          }
        },
        defaultProps: {
          disableTouchRipple: true
        }
      },
      MuiPaper: {
        styleOverrides: {
          root: {
            backgroundImage: "none",
            backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[900] : "#fff",
            "&[href]": {
              textDecorationLine: "none"
            }
          },
          outlined: {
            display: "block",
            borderColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[500] : theme.palette.grey[200],
            ...theme.palette.mode === "dark" && {
              backgroundColor: theme.palette.primaryDark[700]
            },
            "a&, button&": {
              "&:hover": {
                boxShadow: `0px 4px 20px ${theme.palette.mode === "dark" ? "rgba(0, 0, 0, 0.5)" : "rgba(170, 180, 190, 0.3)"}`
              }
            }
          }
        }
      },
      MuiTableCell: {
        styleOverrides: {
          root: {
            padding: theme.spacing(1, 2),
            borderColor: theme.palette.divider
          },
          head: {
            color: theme.palette.text.primary,
            fontWeight: 700
          },
          body: {
            color: theme.palette.text.secondary
          }
        }
      },
      MuiToggleButtonGroup: {
        defaultProps: {
          size: "small"
        }
      },
      MuiToggleButton: {
        styleOverrides: {
          root: {
            textTransform: "none",
            fontWeight: 500,
            padding: theme.spacing(0.5),
            color: theme.palette.mode === "dark" ? theme.palette.grey[300] : theme.palette.grey[700],
            borderColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[500] : theme.palette.grey[200],
            "&.Mui-selected": {
              color: theme.palette.mode === "dark" ? "#fff" : theme.palette.primary[500],
              borderColor: theme.palette.mode === "dark" ? `${theme.palette.primary[700]} !important` : `${theme.palette.primary[500]} !important`,
              backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.primary[50],
              "&:hover": {
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.primary[100]
              }
            }
          }
        }
      },
      MuiTooltip: {
        styleOverrides: {
          tooltip: {
            padding: "5px 9px"
          }
        }
      },
      MuiSwitch: {
        styleOverrides: {
          root: {
            width: 32,
            height: 20,
            padding: 0,
            "& .MuiSwitch-switchBase": {
              "&.Mui-checked": {
                transform: "translateX(11px)",
                color: "#fff"
              }
            }
          },
          switchBase: {
            height: 20,
            width: 20,
            padding: 0,
            color: "#fff",
            "&.Mui-checked + .MuiSwitch-track": {
              opacity: 1
            }
          },
          track: {
            opacity: 1,
            borderRadius: 32,
            backgroundColor: theme.palette.mode === "dark" ? theme.palette.grey[800] : theme.palette.grey[400]
          },
          thumb: {
            flexShrink: 0,
            width: "14px",
            height: "14px"
          }
        }
      },
      MuiPaginationItem: {
        styleOverrides: {
          root: {
            textTransform: "none",
            fontWeight: 700,
            color: theme.palette.mode === "dark" ? theme.palette.grey[300] : theme.palette.grey[700],
            borderColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[500] : theme.palette.grey[200],
            "&.Mui-selected": {
              color: theme.palette.mode === "dark" ? "#fff" : theme.palette.primary[500],
              borderColor: theme.palette.mode === "dark" ? `${theme.palette.primary[700]} !important` : `${theme.palette.primary[500]} !important`,
              backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[700] : theme.palette.primary[50],
              "&:hover": {
                backgroundColor: theme.palette.mode === "dark" ? theme.palette.primaryDark[600] : theme.palette.primary[100]
              }
            }
          }
        }
      },
      MuiCssBaseline: {
        defaultProps: {
          enableColorScheme: true
        }
      },
      MuiTextField: {
        defaultProps: {
          size: "small",
          margin: "dense"
        }
      },
      MuiCheckbox: {
        defaultProps: {
          size: "small"
        }
      },
      MuiFormControl: {
        defaultProps: {
          size: "small",
          margin: "dense"
        }
      },
      MuiFormHelperText: {
        defaultProps: {
          margin: "dense"
        }
      },
      MuiMenuItem: {
        defaultProps: {
          dense: true
        }
      },
      MuiAutocomplete: {
        defaultProps: {
          size: "small"
        }
      },
      MuiDataGrid: {
        defaultProps: {
          density: "compact"
        }
      },
      MuiSvgIcon: {
        defaultProps: {
          fontSize: "small"
        }
      },
      MuiTabs: {
        styleOverrides: {
          root: {
            minHeight: 0
          }
        }
      },
      MuiToolbar: {
        defaultProps: {
          variant: "dense"
        }
      },
      MuiFilledInput: {
        defaultProps: {
          margin: "dense"
        }
      },
      MuiInputBase: {
        defaultProps: {
          margin: "dense"
        }
      },
      MuiInputLabel: {
        defaultProps: {
          margin: "dense"
        }
      },
      MuiListItem: {
        defaultProps: {
          dense: true
        }
      },
      MuiOutlinedInput: {
        defaultProps: {
          margin: "dense"
        }
      },
      MuiFab: {
        defaultProps: {
          size: "small"
        }
      },
      MuiTable: {
        defaultProps: {
          size: "small"
        }
      }
    }
  };
}

// src/utils/useLocalStorageState.ts
import * as React8 from "react";
import useStorageState from "@mui/toolpad-utils/hooks/useStorageState";
function useLocalStorageState(key, initialValue) {
  const [input, setInput] = useStorageState("local", key, () => JSON.stringify(initialValue));
  const value = React8.useMemo(() => JSON.parse(input), [input]);
  const handleChange = React8.useCallback(
    (newValue) => setInput(
      JSON.stringify(typeof newValue === "function" ? newValue(value) : newValue)
    ),
    [setInput, value]
  );
  return [value, handleChange];
}

// src/ThemeContext.tsx
function usePreferredMode() {
  const prefersDarkMode = useMediaQuery("(prefers-color-scheme: dark)");
  return prefersDarkMode ? "dark" : "light";
}
function useThemeMode() {
  const [themeMode, setThemeMode] = useLocalStorageState(
    "toolpad-palette-mode",
    "system"
  );
  return { themeMode, setThemeMode };
}
function usePaletteMode() {
  const preferredMode = usePreferredMode();
  const { themeMode } = useThemeMode();
  return themeMode === "system" ? preferredMode : themeMode;
}
function ThemeProvider2({ children }) {
  const paletteMode = usePaletteMode();
  const theme = React9.useMemo(() => {
    const brandingDesignTokens = getDesignTokens(paletteMode);
    let nextTheme = createTheme3({
      ...brandingDesignTokens,
      palette: {
        ...brandingDesignTokens.palette,
        mode: paletteMode
      }
    });
    nextTheme = deepmerge2(nextTheme, getThemedComponents(nextTheme));
    return nextTheme;
  }, [paletteMode]);
  React9.useMemo(() => {
    let meta = document.querySelector("meta[name='theme-color']");
    if (!meta) {
      meta = document.createElement("meta");
      meta.name = "theme-color";
      document.head.appendChild(meta);
    }
    meta.setAttribute("content", getMetaThemeColor(paletteMode));
    meta.setAttribute("media", `(prefers-color-scheme: ${paletteMode})`);
  }, [paletteMode]);
  return /* @__PURE__ */ React9.createElement(MuiThemeProvider, { theme }, /* @__PURE__ */ React9.createElement(ScopedCssBaseline, { enableColorScheme: true }, children));
}

// src/toolpad/ToolpadShell/Header/index.tsx
function Header({ actions, status, enableUserFeedback = true }) {
  const theme = useTheme();
  const { themeMode, setThemeMode } = useThemeMode();
  const handleThemeModeChange = React10.useCallback(
    (event, mode) => {
      setThemeMode(mode);
    },
    [setThemeMode]
  );
  const productIcon = theme.palette.mode === "dark" ? "/product-icon-dark.svg" : "/product-icon-light.svg";
  return /* @__PURE__ */ React10.createElement(React10.Fragment, null, /* @__PURE__ */ React10.createElement(
    AppBar,
    {
      position: "fixed",
      color: "default",
      elevation: 0,
      sx: { zIndex: theme.zIndex.drawer + 1, borderBottom: 1, borderColor: "divider" }
    },
    /* @__PURE__ */ React10.createElement(Toolbar2, null, /* @__PURE__ */ React10.createElement(
      Box3,
      {
        sx: {
          flex: 1,
          display: "flex",
          flexDirection: "row",
          alignItems: "center",
          justifyContent: "start"
        }
      },
      /* @__PURE__ */ React10.createElement(Tooltip3, { title: "Home" }, /* @__PURE__ */ React10.createElement(
        Link2,
        {
          color: "inherit",
          "aria-label": "Home",
          href: "/",
          underline: "none",
          sx: { display: "flex", flexDirection: "row", alignItems: "center", gap: 1 }
        },
        /* @__PURE__ */ React10.createElement("img", { src: productIcon, alt: "Toolpad product icon", width: 25, height: 25 }),
        /* @__PURE__ */ React10.createElement(
          Box3,
          {
            "data-testid": "brand",
            sx: {
              color: "primary.main",
              lineHeight: "21px",
              fontSize: "16px",
              fontWeight: 700,
              letterSpacing: 0,
              fontFamily: theme.typography.fontFamily
            }
          },
          "MUI Toolpad"
        )
      )),
      /* @__PURE__ */ React10.createElement(Chip2, { sx: { ml: 1 }, label: "Beta", size: "small", color: "grey" })
    ), /* @__PURE__ */ React10.createElement(
      Box3,
      {
        sx: {
          flex: 1,
          display: "flex",
          flexDirection: "row",
          alignItems: "center",
          justifyContent: "center"
        }
      },
      actions
    ), /* @__PURE__ */ React10.createElement(
      Box3,
      {
        sx: {
          flex: 1,
          display: "flex",
          flexDirection: "row",
          alignItems: "center",
          justifyContent: "end",
          gap: 2
        }
      },
      status,
      /* @__PURE__ */ React10.createElement(ThemeModeMenu_default, { mode: themeMode, onChange: handleThemeModeChange }),
      enableUserFeedback ? /* @__PURE__ */ React10.createElement(UserFeedback_default, null) : null
    ))
  ), /* @__PURE__ */ React10.createElement(Toolbar2, null));
}
var Header_default = Header;

// src/runtime/PreviewHeader.tsx
function PreviewHeader({ pageId }) {
  return /* @__PURE__ */ React11.createElement(ThemeProvider2, null, /* @__PURE__ */ React11.createElement(
    Header_default,
    {
      enableUserFeedback: false,
      actions: /* @__PURE__ */ React11.createElement(Stack, { direction: "row", spacing: 1, alignItems: "center" }, /* @__PURE__ */ React11.createElement(Typography, { variant: "body2", sx: { color: "primary.main" } }, "This is a preview version of the application."), /* @__PURE__ */ React11.createElement(
        Button2,
        {
          variant: "outlined",
          endIcon: /* @__PURE__ */ React11.createElement(EditIcon, null),
          color: "primary",
          component: "a",
          href: pageId ? `/_toolpad/app/pages/${pageId}` : "/_toolpad/app"
        },
        "Edit"
      ))
    }
  ));
}

// src/utils/useEvent.ts
import * as React12 from "react";
var dispatcher = null;
function getCurrentDispatcher() {
  return React12.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentDispatcher.current;
}
function useRenderTracker() {
  if (dispatcher === null) {
    dispatcher = getCurrentDispatcher();
  }
}
function isInRender() {
  return dispatcher !== null && dispatcher === getCurrentDispatcher();
}
function useEvent(handler) {
  useRenderTracker();
  const ref = React12.useRef(handler);
  React12.useInsertionEffect(() => {
    ref.current = handler;
  });
  return React12.useCallback((...args) => {
    if (process.env.NODE_ENV !== "production" && isInRender()) {
      console.error(`Functions returned by useEvent can't be called during a React render.`);
    }
    const fn = ref.current;
    fn(...args);
  }, []);
}

// src/runtime/ToolpadApp.tsx
var browserJsRuntime = getBrowserRuntime();
var isPreview = process.env.NODE_ENV !== "production";
var isRenderedInCanvas = typeof window === "undefined" ? false : !!window.frameElement?.dataset?.toolpadCanvas;
var Pre = styled("pre")(({ theme }) => ({
  margin: 0,
  fontFamily: theme.fontFamilyMonospaced
}));
var PREVIEW_PAGE_ROUTE = "/preview/pages/:nodeId";
var internalComponents = Object.fromEntries(
  [...INTERNAL_COMPONENTS].map(([name]) => {
    let builtIn = builtIns[name];
    if (!isToolpadComponent(builtIn)) {
      builtIn = createToolpadComponentThatThrows(
        new Error(`Imported builtIn "${name}" is not a ToolpadComponent`)
      );
    }
    return [name, builtIn];
  })
);
var ReactQueryDevtoolsProduction = React13.lazy(
  () => import("@tanstack/react-query-devtools/build/lib/index.prod.js").then((d) => ({
    default: d.ReactQueryDevtools
  }))
);
var EMPTY_ARRAY2 = [];
var EMPTY_OBJECT2 = {};
var INITIAL_FETCH = {
  call: async () => {
  },
  refetch: async () => {
  },
  fetch: async () => {
  },
  isLoading: false,
  isFetching: false,
  error: null,
  data: null,
  rows: []
};
function usePageNavigator() {
  const navigate = useNavigate();
  const canvasEvents = React13.useContext(CanvasEventsContext);
  const navigateToPage = React13.useCallback(
    (pageNodeId, pageParameters) => {
      const urlParams = pageParameters && new URLSearchParams(pageParameters);
      if (canvasEvents) {
        canvasEvents.emit("pageNavigationRequest", { pageNodeId });
      } else {
        navigate({
          pathname: `/pages/${pageNodeId}`,
          ...urlParams ? {
            search: urlParams.toString()
          } : {}
        });
      }
    },
    [canvasEvents, navigate]
  );
  return navigateToPage;
}
function isEqual(a, b) {
  return a.value === b.value && !!a.error === !!b.error && a.loading === b.loading;
}
var AppRoot = styled("div")({
  overflow: "auto",
  position: "relative",
  minHeight: "100vh",
  display: "flex",
  flexDirection: "column"
});
var EditorOverlay = styled("div")({
  position: "absolute",
  inset: "0 0 0 0",
  pointerEvents: "none",
  overflow: "hidden"
});
function createScope(id, bindings, { localValues, parentScope, meta = {} }) {
  const scopeValues = { ...parentScope?.values, ...localValues };
  const evaluatedBindings = evalJsBindings(browserJsRuntime, bindings, scopeValues);
  return {
    id,
    parentScope,
    bindings: mapValues3(evaluatedBindings, (binding) => binding.result || { value: void 0 }),
    values: buildGlobalScope(scopeValues, evaluatedBindings),
    meta
  };
}
function useApplicationVm(onUpdate) {
  const vm = { scopes: {}, bindingScopes: {} };
  let scheduledUpdate;
  const scheduleUpdate = () => {
    if (scheduledUpdate) {
      return;
    }
    scheduledUpdate = Promise.resolve().then(() => {
      onUpdate(vm);
      scheduledUpdate = void 0;
    });
  };
  return React13.useRef({
    registerScope(scope) {
      if (vm.scopes[scope.id]) {
        throw new Error(`Scope with id "${scope.id}" already registered`);
      }
      vm.scopes[scope.id] = scope;
      scheduleUpdate();
      return () => {
        delete vm.scopes[scope.id];
        scheduleUpdate();
      };
    },
    registerBindingScope(bindingId, scope) {
      if (vm.bindingScopes[bindingId]) {
        return () => {
        };
      }
      vm.bindingScopes[bindingId] = scope.id;
      scheduleUpdate();
      return () => {
        delete vm.bindingScopes[bindingId];
        scheduleUpdate();
      };
    }
  });
}
var ApplicationVmApiContext = React13.createContext(void 0);
var RuntimeScopeContext = React13.createContext(void 0);
var [useDomContext, DomContextProvider] = createProvidedContext("Dom");
var [useEvaluateScopeExpression, EvaluateScopeExpressionProvider] = createProvidedContext("EvaluateScopeExpression");
var SetBindingContext = React13.createContext(void 0);
function getComponentId(elm) {
  const componentId = getElementNodeComponentId(elm);
  return componentId;
}
function flattenNestedBindables(params, prefix = "") {
  if (!params) {
    return [];
  }
  if (Array.isArray(params)) {
    return params.flatMap((param, i) => {
      return flattenNestedBindables(param[1], `${prefix}[${i}][1]`);
    });
  }
  if (typeof params !== "object" || getBindingType(params) !== "const") {
    return [[prefix, params]];
  }
  return Object.entries(params).flatMap(
    ([key, param]) => flattenNestedBindables(param, `${prefix}.${key}`)
  );
}
function resolveBindables(bindings, bindingId, params) {
  const result = {};
  const resultKey = "value";
  const flattened = flattenNestedBindables(params);
  for (const [path] of flattened) {
    const resolvedBinding = bindings[`${bindingId}${path}`];
    if (resolvedBinding?.error) {
      return { error: resolvedBinding?.error };
    }
    if (resolvedBinding?.loading) {
      return { loading: true };
    }
    setObjectPath2(result, `${resultKey}${path}`, resolvedBinding?.value);
  }
  return { value: result[resultKey] || {} };
}
function parseBinding(bindable, { scopePath } = {}) {
  const bindingType = getBindingType(bindable);
  if (bindingType === "const") {
    return {
      scopePath,
      result: { value: bindable }
    };
  }
  if (bindingType === "jsExpression") {
    return {
      scopePath,
      expression: bindable.$$jsExpression
    };
  }
  return {
    scopePath,
    result: { value: void 0 }
  };
}
function getScopeElements(dom, rootNode, components) {
  if (Array.isArray(rootNode)) {
    return [...rootNode, ...rootNode.flatMap((child) => getScopeElements(dom, child, components))];
  }
  const childNodes = getChildNodes(dom, rootNode);
  const result = [];
  for (const [prop, children] of Object.entries(childNodes)) {
    if (isElement(rootNode)) {
      const componentId = getComponentId(rootNode);
      const Component = components[componentId];
      const componentConfig = Component?.[TOOLPAD_COMPONENT];
      const { argTypes = {} } = componentConfig ?? {};
      if (argTypes[prop]?.type !== "template") {
        result.push(
          ...children,
          ...children.flatMap((child) => getScopeElements(dom, child, components))
        );
      }
    } else {
      result.push(
        ...children,
        ...children.flatMap((child) => getScopeElements(dom, child, components))
      );
    }
  }
  return result;
}
function getQueryConfigBindings({ enabled, refetchInterval }) {
  return { enabled, refetchInterval };
}
function parseBindings(dom, rootNode, components, location) {
  const scopeElements = getScopeElements(dom, rootNode, components);
  const parsedBindingsMap = /* @__PURE__ */ new Map();
  const controlled = /* @__PURE__ */ new Set();
  const scopeMeta = {};
  for (const elm of scopeElements) {
    if (isElement(elm)) {
      const componentId = getComponentId(elm);
      const Component = components[componentId];
      const componentConfig = Component?.[TOOLPAD_COMPONENT];
      const { argTypes = {} } = componentConfig ?? {};
      const propsMeta = {};
      for (const [propName, argType] of Object.entries(argTypes)) {
        const initializerId = argType?.defaultValueProp ? `${elm.id}.props.${argType.defaultValueProp}` : void 0;
        const propValue = elm.props?.[propName];
        const binding = propValue ?? (argType ? getArgTypeDefaultValue(argType) : void 0);
        const bindingId = `${elm.id}.props.${propName}`;
        let scopePath;
        const isResizableHeightProp = componentConfig?.resizableHeightProp && propName === componentConfig?.resizableHeightProp;
        if (componentId !== PAGE_ROW_COMPONENT_ID && !isResizableHeightProp && argType?.control?.bindable !== false) {
          scopePath = `${elm.name}.${propName}`;
        }
        propsMeta[propName] = {
          tsType: argType?.tsType
        };
        if (argType) {
          if (argType.onChangeProp) {
            controlled.add(bindingId);
            parsedBindingsMap.set(bindingId, {
              scopePath,
              initializer: initializerId
            });
          } else {
            parsedBindingsMap.set(bindingId, parseBinding(binding, { scopePath }));
          }
        }
        const parseNestedBindings = (value, parentBindingId) => {
          if (value && typeof value === "object") {
            for (const [nestedPropName, nestedProp] of Object.entries(value)) {
              const nestedBindingId = `${parentBindingId}${Array.isArray(value) ? `[${nestedPropName}]` : `.${nestedPropName}`}`;
              if (nestedProp && getBindingType(nestedProp) !== "const") {
                parsedBindingsMap.set(nestedBindingId, parseBinding(nestedProp));
              } else {
                parseNestedBindings(
                  value[nestedPropName],
                  nestedBindingId
                );
              }
            }
          }
        };
        const propBindingValue = propValue && getBindingValue(propValue);
        parseNestedBindings(propBindingValue, bindingId);
      }
      if (componentId !== PAGE_ROW_COMPONENT_ID) {
        scopeMeta[elm.name] = {
          kind: "element",
          componentId,
          props: propsMeta
        };
      }
      if (!isPageLayoutComponent(elm)) {
        for (const [propName, argType] of Object.entries(layoutBoxArgTypes)) {
          const binding = elm.layout?.[propName] ?? (argType ? getArgTypeDefaultValue(argType) : void 0);
          const bindingId = `${elm.id}.layout.${propName}`;
          parsedBindingsMap.set(bindingId, parseBinding(binding, {}));
        }
      }
    }
    if (isQuery(elm)) {
      scopeMeta[elm.name] = {
        kind: "query"
      };
      if (elm.params) {
        const nestedBindablePaths2 = flattenNestedBindables(Object.fromEntries(elm.params ?? []));
        for (const [nestedPath, paramValue] of nestedBindablePaths2) {
          const bindingId = `${elm.id}.params${nestedPath}`;
          const scopePath = `${elm.name}.params${nestedPath}`;
          const bindable = paramValue;
          parsedBindingsMap.set(bindingId, parseBinding(bindable, { scopePath }));
        }
      }
      for (const [key, value] of Object.entries(INITIAL_FETCH)) {
        const bindingId = `${elm.id}.${key}`;
        const scopePath = `${elm.name}.${key}`;
        controlled.add(bindingId);
        parsedBindingsMap.set(bindingId, {
          scopePath,
          result: { value, loading: true }
        });
      }
      const configBindings = getQueryConfigBindings(elm.attributes);
      const nestedBindablePaths = flattenNestedBindables(configBindings);
      for (const [nestedPath, paramValue] of nestedBindablePaths) {
        const bindingId = `${elm.id}.config${nestedPath}`;
        const scopePath = `${elm.name}.config${nestedPath}`;
        const bindable = paramValue;
        parsedBindingsMap.set(bindingId, parseBinding(bindable, { scopePath }));
      }
    }
    if (isMutation(elm)) {
      if (elm.params) {
        for (const [paramName, bindable] of Object.entries(elm.params)) {
          const bindingId = `${elm.id}.params.${paramName}`;
          const scopePath = `${elm.name}.params.${paramName}`;
          const bindingType = getBindingType(bindable);
          if (bindingType === "const") {
            parsedBindingsMap.set(bindingId, {
              scopePath,
              result: { value: bindable }
            });
          } else if (bindingType === "jsExpression") {
            parsedBindingsMap.set(bindingId, {
              scopePath,
              expression: bindable.$$jsExpression
            });
          }
        }
      }
      for (const [key, value] of Object.entries(INITIAL_FETCH)) {
        const bindingId = `${elm.id}.${key}`;
        const scopePath = `${elm.name}.${key}`;
        controlled.add(bindingId);
        parsedBindingsMap.set(bindingId, {
          scopePath,
          result: { value, loading: true }
        });
      }
    }
  }
  if (!Array.isArray(rootNode) && isPage(rootNode)) {
    if (location && !Array.isArray(rootNode) && isPage(rootNode)) {
      const urlParams = new URLSearchParams(location.search);
      const pageParameters = rootNode.attributes.parameters || [];
      for (const [paramName, paramDefault] of pageParameters) {
        const bindingId = `${rootNode.id}.parameters.${paramName}`;
        const scopePath = `page.parameters.${paramName}`;
        parsedBindingsMap.set(bindingId, {
          scopePath,
          result: { value: urlParams.get(paramName) || paramDefault }
        });
      }
    }
  }
  const parsedBindings = Object.fromEntries(parsedBindingsMap);
  return { parsedBindings, controlled, scopeMeta };
}
function useElmToolpadComponent(elm) {
  const componentId = getElementNodeComponentId(elm);
  return useComponent(componentId);
}
function RenderedNode({ nodeId }) {
  const dom = useDomContext();
  const node = getNode(dom, nodeId, "element");
  const Component = useElmToolpadComponent(node);
  const childNodeGroups = getChildNodes(dom, node);
  return (
    /* eslint-disable-next-line @typescript-eslint/no-use-before-define */
    /* @__PURE__ */ React13.createElement(RenderedNodeContent, { node, childNodeGroups, Component })
  );
}
function RuntimeScoped({
  id,
  parseBindingsResult,
  localScope,
  onUpdate,
  children
}) {
  const parentScope = React13.useContext(RuntimeScopeContext);
  const dom = useDomContext();
  const { parsedBindings, controlled, scopeMeta } = parseBindingsResult;
  const [scopeBindings, setScopeBindings] = React13.useState(parsedBindings);
  const prevDom = React13.useRef(dom);
  React13.useEffect(() => {
    if (dom === prevDom.current) {
      return;
    }
    prevDom.current = dom;
    setScopeBindings((existingBindings) => {
      const updated = {};
      for (const [key, binding] of Object.entries(parsedBindings)) {
        updated[key] = controlled.has(key) ? existingBindings[key] || binding : binding;
      }
      return updated;
    });
  }, [parsedBindings, controlled, dom]);
  const { setBindingByScopePath: setParentBindingByScopePath } = React13.useContext(SetBindingContext) ?? {};
  const setBinding = React13.useCallback(
    (bindingId, result) => {
      setScopeBindings((existingBindings) => {
        const { expression, initializer, ...parsedBinding } = parsedBindings[bindingId];
        const existingBinding = existingBindings[bindingId];
        if (existingBinding?.result && isEqual(existingBinding.result, result)) {
          return existingBindings;
        }
        return {
          ...existingBindings,
          ...{
            [bindingId]: { ...parsedBinding, result }
          }
        };
      });
    },
    [parsedBindings]
  );
  const bindingsByScopePath = React13.useMemo(
    () => new Map(
      Object.entries(scopeBindings).map(
        ([bindingId, binding]) => [binding.scopePath, bindingId]
      )
    ),
    [scopeBindings]
  );
  const setBindingByScopePath = React13.useCallback(
    (scopePath, result) => {
      const bindingToUpdateId = bindingsByScopePath.get(scopePath);
      if (bindingToUpdateId) {
        setBinding(bindingToUpdateId, result);
      } else if (setParentBindingByScopePath) {
        setParentBindingByScopePath(scopePath, result);
      } else {
        throw new Error(`No binding found for scope path "${scopePath}"`);
      }
    },
    [bindingsByScopePath, setBinding, setParentBindingByScopePath]
  );
  const setControlledBinding = React13.useCallback(
    (bindingId, result) => {
      if (!controlled.has(bindingId)) {
        throw new Error(`Not a controlled binding "${bindingId}"`);
      }
      setBinding(bindingId, result);
    },
    [controlled, setBinding]
  );
  const childScope = React13.useMemo(
    () => createScope(id, scopeBindings, {
      localValues: localScope,
      parentScope,
      meta: scopeMeta
    }),
    [id, localScope, parentScope, scopeBindings, scopeMeta]
  );
  const vmRef = React13.useContext(ApplicationVmApiContext);
  React13.useEffect(() => {
    if (!vmRef) {
      return () => {
      };
    }
    return vmRef.current.registerScope(childScope);
  }, [vmRef, childScope]);
  const evaluateScopeExpression = React13.useCallback(
    async (expression) => {
      const updates = {};
      const proxify = (obj, scopePathSegments) => {
        return new Proxy(obj, {
          get(target, prop, receiver) {
            if (typeof prop === "symbol") {
              return Reflect.get(target, prop, receiver);
            }
            const result2 = target[prop];
            if (result2 && typeof result2 === "object") {
              return proxify(result2, [...scopePathSegments, prop]);
            }
            return Reflect.get(target, prop, receiver);
          },
          set(target, prop, newValue, receiver) {
            if (typeof prop === "symbol") {
              return Reflect.set(target, prop, newValue, receiver);
            }
            const scopePath = [...scopePathSegments, prop].join(".");
            updates[scopePath] = newValue;
            return Reflect.set(target, prop, newValue, receiver);
          }
        });
      };
      const scopeValues = childScope.values;
      const result = browserJsRuntime.evaluateExpression(expression, proxify(scopeValues, []));
      await result.value;
      for (const [scopePath, newValue] of Object.entries(updates)) {
        setBindingByScopePath(scopePath, { value: newValue });
      }
      return result;
    },
    [childScope.values, setBindingByScopePath]
  );
  React13.useEffect(() => {
    onUpdate?.({
      scopeMeta,
      scope: childScope
    });
  }, [scopeMeta, childScope, onUpdate]);
  const setBindingContext = React13.useMemo(
    () => ({
      setBinding,
      setControlledBinding,
      setBindingByScopePath
    }),
    [setBinding, setControlledBinding, setBindingByScopePath]
  );
  return /* @__PURE__ */ React13.createElement(RuntimeScopeContext.Provider, { value: childScope }, /* @__PURE__ */ React13.createElement(SetBindingContext.Provider, { value: setBindingContext }, /* @__PURE__ */ React13.createElement(EvaluateScopeExpressionProvider, { value: evaluateScopeExpression }, children)));
}
function TemplateScoped({ id, node, localScope, propName, children }) {
  const dom = useDomContext();
  const components = useComponents();
  const location = useLocation2();
  const parseBindingsResult = React13.useMemo(() => {
    const { [propName]: templateChildren = [] } = getChildNodes(dom, node);
    return parseBindings(dom, templateChildren, components, location);
  }, [components, dom, node, propName, location]);
  return /* @__PURE__ */ React13.createElement(RuntimeScoped, { id, parseBindingsResult, localScope }, children);
}
function NodeError({ error }) {
  return /* @__PURE__ */ React13.createElement(Tooltip4, { title: error.message }, /* @__PURE__ */ React13.createElement(
    "span",
    {
      style: {
        display: "inline-flex",
        flexDirection: "row",
        alignItems: "center",
        padding: 8,
        background: "red",
        color: "white"
      }
    },
    /* @__PURE__ */ React13.createElement(ErrorIcon, { color: "inherit", style: { marginRight: 8 } }),
    " Error"
  ));
}
function RenderedNodeContent({ node, childNodeGroups, Component }) {
  const { setControlledBinding } = React13.useContext(SetBindingContext) ?? {};
  invariant4(setControlledBinding, "Node must be rendered in a RuntimeScoped context");
  const nodeId = node.id;
  const componentConfig = Component[TOOLPAD_COMPONENT];
  const {
    argTypes = {},
    errorProp,
    loadingProp,
    loadingPropSource,
    errorPropSource
  } = componentConfig;
  const isLayoutNode = isPage(node) || isElement(node) && isPageLayoutComponent(node);
  const scope = useAssertedContext(RuntimeScopeContext);
  const liveBindings = scope.bindings;
  const boundProps = React13.useMemo(() => {
    const loadingPropSourceSet = loadingPropSource ? new Set(loadingPropSource) : null;
    const errorPropSourceSet = errorPropSource ? new Set(errorPropSource) : null;
    const hookResult = {};
    let error;
    let loading = false;
    for (const [propName, argType] of Object.entries(argTypes)) {
      const bindingId = `${nodeId}.props.${propName}`;
      const binding = liveBindings[bindingId];
      if (binding) {
        hookResult[propName] = binding.value;
        if (binding.loading && (!loadingPropSourceSet || loadingPropSourceSet.has(propName))) {
          loading = true;
        } else if (!error && binding.error && (!errorPropSourceSet || errorPropSourceSet.has(propName))) {
          error = binding.error;
        }
      }
      if (typeof hookResult[propName] === "undefined" && argType) {
        hookResult[propName] = getArgTypeDefaultValue(argType);
      }
    }
    if (error) {
      if (errorProp) {
        hookResult[errorProp] = error;
      } else {
        console.error(errorFrom(error));
      }
    }
    if (loadingProp && loading) {
      hookResult[loadingProp] = true;
    }
    return hookResult;
  }, [argTypes, errorProp, errorPropSource, liveBindings, loadingProp, loadingPropSource, nodeId]);
  const boundLayoutProps = React13.useMemo(() => {
    const hookResult = {};
    for (const [propName, argType] of isLayoutNode ? [] : Object.entries(layoutBoxArgTypes)) {
      const bindingId = `${nodeId}.layout.${propName}`;
      const binding = liveBindings[bindingId];
      if (binding) {
        hookResult[propName] = binding.value;
      }
      if (typeof hookResult[propName] === "undefined" && argType) {
        hookResult[propName] = getArgTypeDefaultValue(argType);
      }
    }
    return hookResult;
  }, [isLayoutNode, liveBindings, nodeId]);
  const onChangeHandlers = React13.useMemo(
    () => mapProperties2(argTypes, ([key, argType]) => {
      if (!argType || !argType.onChangeProp) {
        return null;
      }
      const handler = (param) => {
        const bindingId = `${nodeId}.props.${key}`;
        const value = argType.onChangeHandler ? argType.onChangeHandler(param) : param;
        setControlledBinding(bindingId, { value });
      };
      return [argType.onChangeProp, handler];
    }),
    [argTypes, nodeId, setControlledBinding]
  );
  const navigateToPage = usePageNavigator();
  const evaluateScopeExpression = useEvaluateScopeExpression();
  const eventHandlers = React13.useMemo(() => {
    return mapProperties2(argTypes, ([key, argType]) => {
      if (!argType || argType.type !== "event" || !isElement(node)) {
        return null;
      }
      const action = node.props?.[key];
      if (action?.$$navigationAction) {
        const handler = async () => {
          const { page, parameters = {} } = action.$$navigationAction;
          if (page) {
            const parsedParameterEntries = await Promise.all(
              Object.keys(parameters).map(async (parameterName) => {
                const parameterValue = parameters[parameterName];
                if (parameterValue?.$$jsExpression) {
                  const result = await evaluateScopeExpression(parameterValue.$$jsExpression);
                  return [parameterName, result.value];
                }
                return [parameterName, parameterValue];
              })
            );
            const parsedParameters = Object.fromEntries(parsedParameterEntries);
            navigateToPage(page, parsedParameters);
          }
        };
        return [key, handler];
      }
      if (action?.$$jsExpressionAction) {
        const handler = () => {
          const code = action.$$jsExpressionAction;
          const exprToEvaluate = `(async () => {${code}})()`;
          evaluateScopeExpression(exprToEvaluate);
        };
        return [key, handler];
      }
      return null;
    });
  }, [argTypes, node, navigateToPage, evaluateScopeExpression]);
  const reactChildren = React13.useMemo(() => {
    const result = {};
    for (const [prop, childNodes] of Object.entries(childNodeGroups)) {
      result[prop] = childNodes.map((child) => /* @__PURE__ */ React13.createElement(RenderedNode, { key: child.id, nodeId: child.id }));
    }
    return result;
  }, [childNodeGroups]);
  const layoutElementProps = React13.useMemo(() => {
    if (isElement(node) && isPageRow(node)) {
      return {
        layoutColumnSizes: childNodeGroups.children.map((child) => child.layout?.columnSize)
      };
    }
    return {};
  }, [childNodeGroups.children, node]);
  const props = React13.useMemo(() => {
    return {
      ...boundProps,
      ...onChangeHandlers,
      ...eventHandlers,
      ...layoutElementProps,
      ...reactChildren
    };
  }, [boundProps, eventHandlers, layoutElementProps, onChangeHandlers, reactChildren]);
  const previousProps = React13.useRef(props);
  const [hasSetInitialBindings, setHasSetInitialBindings] = React13.useState(false);
  React13.useEffect(() => {
    Object.entries(argTypes).forEach(([key, argType]) => {
      if (!argType?.defaultValueProp) {
        return;
      }
      if (hasSetInitialBindings && previousProps.current[argType.defaultValueProp] === props[argType.defaultValueProp]) {
        return;
      }
      const bindingIdToUpdate = `${nodeId}.props.${key}`;
      setControlledBinding(bindingIdToUpdate, { value: props[argType.defaultValueProp] });
    });
    previousProps.current = props;
    setHasSetInitialBindings(true);
  }, [props, argTypes, nodeId, setControlledBinding, hasSetInitialBindings]);
  const wrappedProps = React13.useMemo(() => {
    const hookResult = { ...props };
    for (const [propName, argType] of Object.entries(argTypes)) {
      const isElement2 = argType?.type === "element";
      const isTemplate = argType?.type === "template";
      if (isElement2 || isTemplate) {
        const value = hookResult[propName];
        let wrappedValue = value;
        if (argType.control?.type === "slots") {
          wrappedValue = /* @__PURE__ */ React13.createElement(Slots, { prop: propName }, value);
        } else if (argType.control?.type === "slot" || argType.control?.type === "layoutSlot") {
          wrappedValue = /* @__PURE__ */ React13.createElement(Placeholder, { prop: propName, hasLayout: argType.control?.type === "layoutSlot" }, value);
        }
        if (isTemplate) {
          assertIsElement(node);
          hookResult[propName] = (key, localScope) => {
            return /* @__PURE__ */ React13.createElement(
              TemplateScoped,
              {
                id: `${node.id}.props.${propName}.${key}`,
                localScope,
                node,
                propName
              },
              wrappedValue
            );
          };
        } else {
          hookResult[propName] = wrappedValue;
        }
      }
    }
    return hookResult;
  }, [argTypes, node, props]);
  const vmRef = React13.useContext(ApplicationVmApiContext);
  React13.useEffect(() => {
    if (!vmRef) {
      return () => {
      };
    }
    const unsubscribers = [];
    for (const propName of Object.keys(argTypes)) {
      const unsubscribe = vmRef.current.registerBindingScope(`${nodeId}.props.${propName}`, scope);
      unsubscribers.push(unsubscribe);
    }
    return () => {
      unsubscribers.forEach((unsubscribe) => unsubscribe());
    };
  }, [nodeId, argTypes, vmRef, scope]);
  return /* @__PURE__ */ React13.createElement(
    NodeRuntimeWrapper,
    {
      nodeId,
      nodeName: node.name,
      componentConfig: Component[TOOLPAD_COMPONENT],
      NodeError
    },
    isLayoutNode ? /* @__PURE__ */ React13.createElement(Component, { ...wrappedProps }) : /* @__PURE__ */ React13.createElement(
      Box4,
      {
        sx: {
          display: "flex",
          alignItems: boundLayoutProps.verticalAlign,
          justifyContent: boundLayoutProps.horizontalAlign
        }
      },
      /* @__PURE__ */ React13.createElement(Component, { ...wrappedProps })
    )
  );
}
function PageRoot({ children }) {
  return /* @__PURE__ */ React13.createElement(Container, null, /* @__PURE__ */ React13.createElement(
    Stack2,
    {
      "data-testid": "page-root",
      direction: "column",
      sx: {
        my: 2,
        gap: 1
      }
    },
    children
  ));
}
var PageRootComponent = createComponent(PageRoot, {
  argTypes: {
    children: {
      type: "element",
      control: { type: "slots" }
    }
  }
});
function QueryNode({ page, node }) {
  const { setControlledBinding } = React13.useContext(SetBindingContext) ?? {};
  invariant4(setControlledBinding, "QueryNode must be rendered in a RuntimeScoped context");
  const { bindings } = useAssertedContext(RuntimeScopeContext);
  const params = resolveBindables(
    bindings,
    `${node.id}.params`,
    Object.fromEntries(node.params ?? [])
  );
  const configBindings = getQueryConfigBindings(node.attributes);
  const options2 = resolveBindables(bindings, `${node.id}.config`, configBindings);
  const inputError = params.error || options2.error;
  const inputIsLoading = params.loading || options2.loading;
  const queryResult = useDataQuery(page, node, params.value, {
    ...options2.value,
    enabled: !inputIsLoading && !inputError
  });
  React13.useEffect(() => {
    const { isLoading: queryIsLoading, error: queryError, data, rows, ...result } = queryResult;
    const error = queryError || inputError;
    const isLoading = queryIsLoading || inputIsLoading;
    for (const [key, value] of Object.entries(result)) {
      const bindingId = `${node.id}.${key}`;
      setControlledBinding(bindingId, { value });
    }
    setControlledBinding(`${node.id}.isLoading`, { value: isLoading });
    setControlledBinding(`${node.id}.error`, {
      value: error ? String(error.message || error) : void 0
    });
    const deferredStatus = { loading: isLoading, error };
    setControlledBinding(`${node.id}.data`, { ...deferredStatus, value: data });
    setControlledBinding(`${node.id}.rows`, { ...deferredStatus, value: rows });
  }, [node.name, node.id, queryResult, setControlledBinding, inputError, inputIsLoading]);
  return null;
}
function MutationNode({ node, page }) {
  const { setControlledBinding } = React13.useContext(SetBindingContext) ?? {};
  invariant4(setControlledBinding, "MutationNode must be rendered in a RuntimeScoped context");
  const { bindings } = useAssertedContext(RuntimeScopeContext);
  const queryId = node.id;
  const { value: params } = resolveBindables(
    bindings,
    `${node.id}.params`,
    Object.fromEntries(node.params ?? [])
  );
  const {
    isLoading,
    data: responseData = EMPTY_OBJECT2,
    error: fetchError,
    mutateAsync
  } = useMutation2(
    async (overrides = {}) => execDataSourceQuery({
      pageName: page.name,
      queryName: node.name,
      params: { ...params, ...overrides }
    }),
    {
      mutationKey: [queryId, params]
    }
  );
  const { data, error: apiError } = responseData;
  const error = apiError || fetchError;
  const mutationResult = React13.useMemo(
    () => ({
      isLoading,
      isFetching: isLoading,
      error,
      data,
      rows: Array.isArray(data) ? data : EMPTY_ARRAY2,
      call: mutateAsync,
      fetch: mutateAsync,
      refetch: () => {
        throw new Error(`refetch is not supported in manual queries`);
      }
    }),
    [isLoading, error, mutateAsync, data]
  );
  React13.useEffect(() => {
    for (const [key, value] of Object.entries(mutationResult)) {
      const bindingId = `${node.id}.${key}`;
      setControlledBinding(bindingId, { value });
    }
  }, [node.id, mutationResult, setControlledBinding]);
  return null;
}
function FetchNode({ node, page }) {
  const mode = node.attributes.mode || "query";
  switch (mode) {
    case "query":
      return /* @__PURE__ */ React13.createElement(QueryNode, { node, page });
    case "mutation":
      return /* @__PURE__ */ React13.createElement(MutationNode, { node, page });
    default:
      throw new Error(`Unrecognized fetch mode "${mode}"`);
  }
}
function RenderedPage({ nodeId }) {
  const dom = useDomContext();
  const page = getNode(dom, nodeId, "page");
  const { children = [], queries = [] } = getChildNodes(dom, page);
  usePageTitle(page.attributes.title);
  const location = useLocation2();
  const components = useComponents();
  const parseBindingsResult = React13.useMemo(
    () => parseBindings(dom, page, components, location),
    [components, dom, location, page]
  );
  const canvasEvents = React13.useContext(CanvasEventsContext);
  const onUpdate = useEvent(({ scope, scopeMeta }) => {
    if (canvasEvents) {
      canvasEvents.emit("pageStateUpdated", {
        pageState: scope.values,
        globalScopeMeta: scopeMeta
      });
      canvasEvents.emit("pageBindingsUpdated", { bindings: scope.bindings });
    }
  });
  const applicationVm = useApplicationVm((vm) => {
    if (canvasEvents) {
      canvasEvents.emit("vmUpdated", { vm });
    }
  });
  return /* @__PURE__ */ React13.createElement(ApplicationVmApiContext.Provider, { value: applicationVm }, /* @__PURE__ */ React13.createElement(RuntimeScoped, { id: "global", parseBindingsResult, onUpdate }, /* @__PURE__ */ React13.createElement(
    RenderedNodeContent,
    {
      node: page,
      childNodeGroups: { children },
      Component: PageRootComponent
    }
  ), queries.map((node) => /* @__PURE__ */ React13.createElement(FetchNode, { key: node.id, page, node }))));
}
function PageNotFound() {
  return /* @__PURE__ */ React13.createElement(
    Container,
    {
      sx: {
        display: "flex",
        alignItems: "center",
        justifyContent: "center",
        flexDirection: "column"
      }
    },
    /* @__PURE__ */ React13.createElement(Typography2, { variant: "h1" }, "Not found"),
    /* @__PURE__ */ React13.createElement(Typography2, null, "The page doesn't exist in this application.")
  );
}
function RenderedPages({ pages, defaultPage }) {
  const defaultPageNavigation = /* @__PURE__ */ React13.createElement(Navigate, { to: `/pages/${defaultPage.id}`, replace: true });
  return /* @__PURE__ */ React13.createElement(Routes, null, pages.map((page) => /* @__PURE__ */ React13.createElement(React13.Fragment, { key: page.id }, /* @__PURE__ */ React13.createElement(
    Route,
    {
      path: `/pages/${page.id}`,
      element: /* @__PURE__ */ React13.createElement(
        RenderedPage,
        {
          nodeId: page.id,
          key: page.id
        }
      )
    }
  ))), pages.map((page) => /* @__PURE__ */ React13.createElement(React13.Fragment, { key: page.id }, /* @__PURE__ */ React13.createElement(
    Route,
    {
      path: `/pages/${page.name}`,
      element: /* @__PURE__ */ React13.createElement(Navigate, { to: `/pages/${page.id}`, replace: true })
    }
  ))), /* @__PURE__ */ React13.createElement(Route, { path: "/pages", element: defaultPageNavigation }), /* @__PURE__ */ React13.createElement(Route, { path: "/", element: defaultPageNavigation }), /* @__PURE__ */ React13.createElement(Route, { path: "*", element: /* @__PURE__ */ React13.createElement(PageNotFound, null) }));
}
var FullPageCentered = styled("div")({
  width: "100%",
  height: "100%",
  display: "flex",
  alignItems: "center",
  justifyContent: "center"
});
function AppLoading() {
  return /* @__PURE__ */ React13.createElement(LinearProgress, null);
}
function AppError({ error }) {
  return /* @__PURE__ */ React13.createElement(FullPageCentered, null, /* @__PURE__ */ React13.createElement(Alert, { severity: "error" }, /* @__PURE__ */ React13.createElement(AlertTitle, null, "Something went wrong"), /* @__PURE__ */ React13.createElement(Pre, null, error.message), /* @__PURE__ */ React13.createElement(Pre, null, error.stack)));
}
var queryClient2 = new QueryClient2({
  defaultOptions: {
    queries: {
      retry: false,
      staleTime: 60 * 1e3
    }
  }
});
function ToolpadAppLayout({ dom, hasShell: hasShellProp = true }) {
  const root = getApp(dom);
  const { pages = [] } = getChildNodes(dom, root);
  const location = useLocation2();
  const { pathname, search } = location;
  const urlParams = React13.useMemo(() => new URLSearchParams(search), [search]);
  const pageMatch = matchPath(PREVIEW_PAGE_ROUTE, `/preview${pathname}`);
  const pageId = pageMatch?.params.nodeId;
  const defaultPage = pages[0];
  const page = pageId ? getMaybeNode(dom, pageId, "page") : defaultPage;
  const displayMode = urlParams.get("toolpad-display") || page?.attributes.display;
  const hasShell = hasShellProp && displayMode !== "standalone";
  const showPreviewHeader = isPreview && !isRenderedInCanvas;
  return /* @__PURE__ */ React13.createElement(React13.Fragment, null, showPreviewHeader ? /* @__PURE__ */ React13.createElement(PreviewHeader, { pageId }) : null, /* @__PURE__ */ React13.createElement(Box4, { sx: { flex: 1, display: "flex" } }, hasShell && pages.length > 0 ? /* @__PURE__ */ React13.createElement(AppNavigation, { pages, clipped: showPreviewHeader }) : null, /* @__PURE__ */ React13.createElement(RenderedPages, { pages, defaultPage })));
}
function ToolpadApp({
  rootRef,
  loadComponents,
  basename,
  hasShell = true,
  state
}) {
  const { dom } = state;
  const [components, setComponents] = React13.useState(null);
  const [resetNodeErrorsKey, setResetNodeErrorsKey] = React13.useState(0);
  React13.useEffect(() => setResetNodeErrorsKey((key) => key + 1), [dom]);
  const { value: showDevtools, toggle: toggleDevtools } = useBoolean(false);
  React13.useEffect(() => {
    window.toggleDevtools = () => toggleDevtools();
  }, [toggleDevtools]);
  React13.useEffect(() => {
    loadComponents(state).then(
      (codeComponents) => setComponents({ ...codeComponents, ...internalComponents })
    );
  }, [loadComponents, state]);
  return /* @__PURE__ */ React13.createElement(AppThemeProvider, { dom }, /* @__PURE__ */ React13.createElement(CssBaseline, { enableColorScheme: true }), /* @__PURE__ */ React13.createElement(AppRoot, { ref: rootRef }, components ? /* @__PURE__ */ React13.createElement(ComponentsContextProvider, { value: components }, /* @__PURE__ */ React13.createElement(DomContextProvider, { value: dom }, /* @__PURE__ */ React13.createElement(ErrorBoundary, { FallbackComponent: AppError }, /* @__PURE__ */ React13.createElement(ResetNodeErrorsKeyProvider, { value: resetNodeErrorsKey }, /* @__PURE__ */ React13.createElement(React13.Suspense, { fallback: /* @__PURE__ */ React13.createElement(AppLoading, null) }, /* @__PURE__ */ React13.createElement(QueryClientProvider, { client: queryClient2 }, /* @__PURE__ */ React13.createElement(BrowserRouter, { basename }, /* @__PURE__ */ React13.createElement(ToolpadAppLayout, { dom, hasShell })), showDevtools ? /* @__PURE__ */ React13.createElement(ReactQueryDevtoolsProduction, { initialIsOpen: false }) : null)))))) : /* @__PURE__ */ React13.createElement(AppLoading, null), /* @__PURE__ */ React13.createElement(EditorOverlay, { id: HTML_ID_EDITOR_OVERLAY })));
}

export {
  CanvasHooksContext,
  TOOLPAD_BRIDGE_GLOBAL,
  queryClient2 as queryClient,
  ToolpadApp
};
//# sourceMappingURL=chunk-UTK5C7AF.mjs.map