import {
  FORM_INPUT_ARG_TYPES,
  FORM_TEXT_INPUT_ARG_TYPES,
  useFormInput,
  withComponentForm
} from "./chunk-44V7URYC.js";
import {
  SX_PROP_HELPER_TEXT
} from "./chunk-Z6S6EGNW.js";

// src/Autocomplete.tsx
import * as React from "react";
import {
  Autocomplete as MuiAutocomplete,
  TextField
} from "@mui/material";
import { createComponent } from "@mui/toolpad-core";
function Autocomplete({
  options,
  label,
  onChange,
  value,
  isRequired,
  minLength,
  maxLength,
  isInvalid,
  sx,
  ...rest
}) {
  const [selectedVal, setSelectedVal] = React.useState(null);
  const { onFormInputChange, formInputError, renderFormInput } = useFormInput({
    name: rest.name,
    label,
    value,
    onChange,
    emptyValue: null,
    validationProps: { isRequired, minLength, maxLength, isInvalid }
  });
  const getValue = React.useCallback((selection) => {
    if (!selection) {
      return null;
    }
    if (typeof selection === "string") {
      return selection;
    }
    if (typeof selection === "object") {
      return selection?.value ?? selection?.label ?? null;
    }
    return null;
  }, []);
  const getOptionLabel = React.useCallback((option) => {
    if (!option) {
      return "";
    }
    if (typeof option === "string") {
      return option;
    }
    if (typeof option === "object") {
      return option?.label ?? "";
    }
    return "";
  }, []);
  const handleChange = React.useCallback(
    (event, selection) => {
      const newValue = getValue(selection);
      onFormInputChange(newValue);
      setSelectedVal(selection);
    },
    [getValue, onFormInputChange]
  );
  React.useEffect(() => {
    if (!value) {
      setSelectedVal(null);
    }
  }, [value]);
  return renderFormInput(
    /* @__PURE__ */ React.createElement(
      MuiAutocomplete,
      {
        onChange: handleChange,
        options: options ?? [],
        isOptionEqualToValue: (option, selectedValue) => getValue(option) === getValue(selectedValue),
        getOptionLabel,
        value: selectedVal,
        renderInput: (params) => /* @__PURE__ */ React.createElement(
          TextField,
          {
            ...params,
            label,
            variant: "outlined",
            ...formInputError && {
              error: Boolean(formInputError),
              helperText: formInputError.message || ""
            }
          }
        ),
        sx: { ...!rest.fullWidth && !value ? { width: 120 } : {}, ...sx },
        ...rest
      }
    )
  );
}
var FormWrappedAutocomplete = withComponentForm(Autocomplete);
var Autocomplete_default = createComponent(FormWrappedAutocomplete, {
  layoutDirection: "both",
  loadingProp: "loading",
  argTypes: {
    options: {
      helperText: "The options available to search from.",
      type: "array",
      default: [],
      control: {
        type: "SelectOptions"
      }
    },
    value: {
      helperText: "The value of the autocomplete.",
      type: "string",
      onChangeProp: "onChange",
      default: ""
    },
    label: {
      helperText: "The label to display for the autocomplete.",
      type: "string",
      default: "Search\u2026"
    },
    fullWidth: {
      helperText: "If true, the autocomplete will take up the full width of its container.",
      type: "boolean",
      default: true
    },
    size: {
      helperText: "The size of the autocomplete. One of `small`, `medium`, or `large`.",
      type: "string",
      enum: ["small", "medium", "large"],
      default: "small"
    },
    loading: {
      helperText: "If true, the autocomplete will display a loading indicator.",
      type: "boolean"
    },
    disabled: {
      helperText: "If true, the autocomplete will be disabled.",
      type: "boolean"
    },
    ...FORM_INPUT_ARG_TYPES,
    ...FORM_TEXT_INPUT_ARG_TYPES,
    sx: {
      helperText: SX_PROP_HELPER_TEXT,
      type: "object"
    }
  }
});

export {
  Autocomplete_default
};
//# sourceMappingURL=chunk-6IIDTBJO.js.map