import {
  ErrorOverlay
} from "./chunk-NFE6IAQR.js";
import {
  SX_PROP_HELPER_TEXT
} from "./chunk-Z6S6EGNW.js";

// src/DataGrid.tsx
import {
  DataGridPro,
  GridToolbar,
  useGridApiContext,
  gridColumnsTotalWidthSelector,
  gridColumnPositionsSelector,
  useGridApiRef,
  useGridRootProps,
  gridDensityFactorSelector,
  useGridSelector,
  getGridDefaultColumnTypes
} from "@mui/x-data-grid-pro";
import {
  Unstable_LicenseInfoProvider as LicenseInfoProvider
} from "@mui/x-license-pro";
import * as React from "react";
import { useNode, createComponent, useComponents } from "@mui/toolpad-core";
import {
  Box,
  debounce,
  LinearProgress,
  Skeleton,
  Link,
  styled,
  Typography,
  Tooltip,
  Popover
} from "@mui/material";
import { getObjectKey } from "@mui/toolpad-utils/objectKey";
import { errorFrom } from "@mui/toolpad-utils/errors";
import { hasImageExtension } from "@mui/toolpad-utils/path";
import { ErrorBoundary } from "react-error-boundary";
import { createStringFormatter } from "@mui/toolpad-core/numberFormat";
var LICENSE_INFO = {
  key: "f359d9c0d105599a7d83c3f8d775eca5Tz0xMjMsRT0yNTI0NjA0NDAwMDAwLFM9cHJlbWl1bSxMTT1wZXJwZXR1YWwsS1Y9Mg=="
};
var DEFAULT_COLUMN_TYPES = getGridDefaultColumnTypes();
function mulberry32(a) {
  return () => {
    let t = a += 1831565813;
    t = Math.imul(t ^ t >>> 15, t | 1);
    t ^= t + Math.imul(t ^ t >>> 7, t | 61);
    return ((t ^ t >>> 14) >>> 0) / 4294967296;
  };
}
function randomBetween(seed, min, max) {
  const random = mulberry32(seed);
  return () => min + (max - min) * random();
}
var SkeletonCell = styled(Box)(({ theme }) => ({
  display: "flex",
  flexDirection: "row",
  alignItems: "center",
  borderBottom: `1px solid ${theme.palette.divider}`
}));
function SkeletonLoadingOverlay() {
  const apiRef = useGridApiContext();
  const rootProps = useGridRootProps();
  const dimensions = apiRef.current?.getRootDimensions();
  const viewportHeight = dimensions?.viewportInnerSize.height ?? 0;
  const factor = useGridSelector(apiRef, gridDensityFactorSelector);
  const rowHeight = Math.floor(rootProps.rowHeight * factor);
  const skeletonRowsCount = Math.ceil(viewportHeight / rowHeight);
  const totalWidth = gridColumnsTotalWidthSelector(apiRef);
  const positions = gridColumnPositionsSelector(apiRef);
  const inViewportCount = React.useMemo(
    () => positions.filter((value) => value <= totalWidth).length,
    [totalWidth, positions]
  );
  const columns = apiRef.current.getVisibleColumns().slice(0, inViewportCount);
  const children = React.useMemo(() => {
    const random = randomBetween(12345, 25, 75);
    const array = [];
    for (let i = 0; i < skeletonRowsCount; i += 1) {
      for (const column of columns) {
        const width = Math.round(random());
        array.push(
          /* @__PURE__ */ React.createElement(SkeletonCell, { key: `col-${column.field}-${i}`, sx: { justifyContent: column.align } }, /* @__PURE__ */ React.createElement(Skeleton, { sx: { mx: 1 }, width: `${width}%` }))
        );
      }
      array.push(/* @__PURE__ */ React.createElement(SkeletonCell, { key: `fill-${i}` }));
    }
    return array;
  }, [skeletonRowsCount, columns]);
  const rowsCount = apiRef.current.getRowsCount();
  return rowsCount > 0 ? /* @__PURE__ */ React.createElement(LinearProgress, null) : /* @__PURE__ */ React.createElement(
    "div",
    {
      style: {
        display: "grid",
        gridTemplateColumns: `${columns.map(({ computedWidth }) => `${computedWidth}px`).join(" ")} 1fr`,
        gridAutoRows: `${rowHeight}px`
      }
    },
    children
  );
}
function inferColumnType(value) {
  if (value instanceof Date) {
    return "dateTime";
  }
  const valueType = typeof value;
  switch (typeof value) {
    case "number":
    case "boolean":
      return valueType;
    case "string":
      try {
        const url = new URL(value);
        if (hasImageExtension(url.pathname)) {
          return "image";
        }
        return "link";
      } catch (error) {
        return valueType;
      }
    case "object":
      return "json";
    default:
      return "string";
  }
}
function ImageCell({ field, id, value: src }) {
  const [anchorEl, setAnchorEl] = React.useState(null);
  const handlePopoverOpen = (event) => {
    setAnchorEl(event.currentTarget);
  };
  const handlePopoverClose = () => {
    setAnchorEl(null);
  };
  const open = Boolean(anchorEl);
  const popoverId = React.useId();
  const alt = `${field} ${id}`;
  return /* @__PURE__ */ React.createElement(React.Fragment, null, /* @__PURE__ */ React.createElement(
    Box,
    {
      "aria-owns": open ? popoverId : void 0,
      "aria-haspopup": "true",
      onMouseEnter: handlePopoverOpen,
      onMouseLeave: handlePopoverClose,
      component: "img",
      src,
      alt,
      sx: { maxWidth: "100%", maxHeight: "100%", objectFit: "contain", py: 1 }
    }
  ), /* @__PURE__ */ React.createElement(
    Popover,
    {
      id: popoverId,
      sx: {
        pointerEvents: "none"
      },
      open,
      anchorEl,
      onClose: handlePopoverClose,
      disableRestoreFocus: true
    },
    /* @__PURE__ */ React.createElement(
      Box,
      {
        component: "img",
        src,
        alt,
        sx: { maxWidth: "60vw", maxHeight: "60vh", objectFit: "contain", m: 1 }
      }
    )
  ));
}
function dateValueGetter({ value }) {
  return typeof value === "number" ? new Date(value) : value;
}
function ComponentErrorFallback({ error }) {
  return /* @__PURE__ */ React.createElement(Typography, { variant: "overline", sx: { color: "error.main", fontSize: "10px" } }, "Code component error", " ", /* @__PURE__ */ React.createElement(Tooltip, { title: error.message }, /* @__PURE__ */ React.createElement("span", null, "\u2139\uFE0F")));
}
function CustomColumn({ params }) {
  const { value, colDef, row, field } = params;
  const column = colDef;
  const components = useComponents();
  const Component = components[`codeComponent.${column.codeComponent}`];
  if (!Component) {
    return /* @__PURE__ */ React.createElement(Typography, { variant: "overline", sx: { color: "error.main", fontSize: "10px" } }, "No component selected");
  }
  return /* @__PURE__ */ React.createElement(ErrorBoundary, { FallbackComponent: ComponentErrorFallback }, /* @__PURE__ */ React.createElement(Component, { value, row, field }));
}
var CUSTOM_COLUMN_TYPES = {
  json: {
    valueFormatter: ({ value: cellValue }) => JSON.stringify(cellValue)
  },
  date: {
    extendType: "date",
    valueGetter: dateValueGetter
  },
  dateTime: {
    extendType: "date",
    valueGetter: dateValueGetter
  },
  link: {
    renderCell: ({ value }) => /* @__PURE__ */ React.createElement(Link, { href: value, target: "_blank", rel: "noopener noreferrer nofollow" }, value)
  },
  image: {
    renderCell: ({ value, ...params }) => value ? /* @__PURE__ */ React.createElement(ImageCell, { value, ...params }) : ""
  },
  codeComponent: {
    renderCell: (params) => {
      return /* @__PURE__ */ React.createElement(CustomColumn, { params });
    }
  }
};
function inferColumns(rows) {
  if (rows.length < 1) {
    return [];
  }
  const firstRow = rows[0];
  return Object.entries(firstRow).map(([field, value]) => {
    return {
      field,
      type: inferColumnType(value)
    };
  });
}
function parseColumns(columns) {
  return columns.map((column) => {
    if (column.type === "number" && column.numberFormat) {
      return {
        ...column,
        valueFormatter: createStringFormatter(column.numberFormat)
      };
    }
    const customType = column.type ? CUSTOM_COLUMN_TYPES[column.type] : {};
    const type = column.type && column.type in DEFAULT_COLUMN_TYPES ? column.type : void 0;
    return { ...customType, ...column, type };
  });
}
var EMPTY_ROWS = [];
var DataGridComponent = React.forwardRef(function DataGridComponent2({
  columns: columnsProp,
  rows: rowsProp,
  height: heightProp,
  rowIdField: rowIdFieldProp,
  error: errorProp,
  selection,
  onSelectionChange,
  hideToolbar,
  ...props
}, ref) {
  const nodeRuntime = useNode();
  const handleResize = React.useMemo(
    () => debounce((params) => {
      if (!nodeRuntime) {
        return;
      }
      nodeRuntime.updateAppDomConstProp(
        "columns",
        (columns2) => columns2?.map(
          (column) => column.field === params.colDef.field ? { ...column, width: params.width } : column
        )
      );
    }, 500),
    [nodeRuntime]
  );
  React.useEffect(() => handleResize.clear(), [handleResize]);
  const handleColumnOrderChange = React.useMemo(
    () => debounce((params) => {
      if (!nodeRuntime) {
        return;
      }
      nodeRuntime.updateAppDomConstProp("columns", (columns2) => {
        if (!columns2) {
          return columns2;
        }
        const old = columns2.find((colDef) => colDef.field === params.column.field);
        if (!old) {
          return columns2;
        }
        const withoutOld = columns2.filter((column) => column.field !== params.column.field);
        return [
          ...withoutOld.slice(0, params.targetIndex),
          old,
          ...withoutOld.slice(params.targetIndex)
        ];
      });
    }, 500),
    [nodeRuntime]
  );
  React.useEffect(() => handleColumnOrderChange.clear(), [handleColumnOrderChange]);
  const rowsInput = rowsProp || EMPTY_ROWS;
  const hasExplicitRowId = React.useMemo(() => {
    const hasRowIdField = !!(rowIdFieldProp && rowIdFieldProp !== "id");
    const parsedRows = rowsInput;
    return parsedRows.length === 0 || hasRowIdField || !!parsedRows[0].id;
  }, [rowIdFieldProp, rowsInput]);
  const rows = React.useMemo(
    () => hasExplicitRowId ? rowsInput : rowsInput.map((row, id) => ({ ...row, id })),
    [hasExplicitRowId, rowsInput]
  );
  const columnsInitRef = React.useRef(false);
  const hasColumnsDefined = columnsProp && columnsProp.length > 0;
  React.useEffect(() => {
    if (!nodeRuntime || hasColumnsDefined || rows.length <= 0 || columnsInitRef.current) {
      return;
    }
    let inferredColumns = inferColumns(rows);
    if (!hasExplicitRowId) {
      inferredColumns = inferredColumns.filter((column) => column.field !== "id");
    }
    nodeRuntime.updateAppDomConstProp("columns", inferredColumns);
    columnsInitRef.current = true;
  }, [hasColumnsDefined, rows, nodeRuntime, hasExplicitRowId]);
  const getRowId = React.useCallback(
    (row) => {
      return rowIdFieldProp && row[rowIdFieldProp] ? row[rowIdFieldProp] : row.id;
    },
    [rowIdFieldProp]
  );
  const onSelectionModelChange = React.useCallback(
    (ids) => {
      onSelectionChange?.(ids.length > 0 ? rows.find((row) => row.id === ids[0]) : null);
    },
    [rows, onSelectionChange]
  );
  const selectionModel = React.useMemo(
    () => selection?.id ? [selection.id] : [],
    [selection?.id]
  );
  const columns = React.useMemo(
    () => columnsProp ? parseColumns(columnsProp) : [],
    [columnsProp]
  );
  const apiRef = useGridApiRef();
  React.useEffect(() => {
    apiRef.current.updateColumns(columns);
  }, [apiRef, columns]);
  const gridKey = React.useMemo(
    () => [getObjectKey(getRowId), getObjectKey(columns)].join("::"),
    [getRowId, columns]
  );
  const error = errorProp ? errorFrom(errorProp) : null;
  return /* @__PURE__ */ React.createElement(LicenseInfoProvider, { info: LICENSE_INFO }, /* @__PURE__ */ React.createElement(
    "div",
    {
      ref,
      style: { height: heightProp, minHeight: "100%", width: "100%", position: "relative" }
    },
    /* @__PURE__ */ React.createElement(ErrorOverlay, { error }),
    /* @__PURE__ */ React.createElement(
      "div",
      {
        style: {
          position: "absolute",
          inset: "0 0 0 0",
          visibility: error ? "hidden" : "visible"
        }
      },
      /* @__PURE__ */ React.createElement(
        DataGridPro,
        {
          apiRef,
          slots: {
            toolbar: hideToolbar ? null : GridToolbar,
            loadingOverlay: SkeletonLoadingOverlay
          },
          onColumnResize: handleResize,
          onColumnOrderChange: handleColumnOrderChange,
          rows,
          columns,
          key: gridKey,
          getRowId,
          onRowSelectionModelChange: onSelectionModelChange,
          rowSelectionModel: selectionModel,
          ...props
        }
      )
    )
  ));
});
var DataGrid_default = createComponent(DataGridComponent, {
  helperText: "The MUI X [Data Grid](https://mui.com/x/react-data-grid/) component.\n\nThe datagrid lets users display tabular data in a flexible grid.",
  errorProp: "error",
  loadingPropSource: ["rows", "columns"],
  loadingProp: "loading",
  resizableHeightProp: "height",
  argTypes: {
    rows: {
      helperText: "The data to be displayed as rows. Must be an array of objects.",
      type: "array",
      schema: {
        type: "array",
        items: {
          type: "object",
          additionalProperties: true,
          properties: {
            id: {
              type: "string"
            }
          },
          required: ["id"]
        }
      }
    },
    columns: {
      helperText: "The columns to be displayed.",
      type: "array",
      schema: {
        type: "array",
        items: {
          type: "object",
          additionalProperties: true,
          properties: {
            field: {
              type: "string"
            },
            align: {
              type: "string",
              enum: ["center", "right", "left"]
            }
          },
          required: ["field"]
        }
      },
      control: { type: "GridColumns", bindable: false }
    },
    rowIdField: {
      helperText: "Defines which column contains the [id](https://mui.com/x/react-data-grid/row-definition/#row-identifier) that uniquely identifies each row.",
      type: "string",
      control: { type: "RowIdFieldSelect" },
      label: "Id field"
    },
    selection: {
      helperText: "The currently selected row. Or `null` in case no row has been selected.",
      type: "object",
      default: null,
      onChangeProp: "onSelectionChange",
      tsType: `ThisComponent['rows'][number] | undefined`
    },
    density: {
      helperText: "The [density](https://mui.com/x/react-data-grid/accessibility/#density-prop) of the rows. Possible values are `compact`, `standard`, or `comfortable`.",
      type: "string",
      enum: ["compact", "standard", "comfortable"],
      default: "compact"
    },
    height: {
      type: "number",
      default: 350,
      minimum: 100
    },
    loading: {
      helperText: "Displays a loading animation indicating the datagrid isn't ready to present data yet.",
      type: "boolean"
    },
    hideToolbar: {
      helperText: "Hide the toolbar area that contains the data grid user controls.",
      type: "boolean"
    },
    sx: {
      helperText: SX_PROP_HELPER_TEXT,
      type: "object"
    }
  }
});

export {
  CUSTOM_COLUMN_TYPES,
  inferColumns,
  parseColumns,
  DataGrid_default
};
//# sourceMappingURL=chunk-DUWNHWQS.js.map