import {
  FORM_INPUT_ARG_TYPES,
  useFormInput,
  withComponentForm
} from "./chunk-44V7URYC.js";

// src/FilePicker.tsx
import * as React from "react";
import { TextField as MuiTextField } from "@mui/material";
import { createComponent } from "@mui/toolpad-core";
var readFile = async (file) => {
  return new Promise((resolve, reject) => {
    const readerBase64 = new FileReader();
    readerBase64.onload = (event) => {
      if (!event.target) {
        reject();
        return;
      }
      resolve(event.target.result);
    };
    readerBase64.readAsDataURL(file);
  });
};
function FilePicker({
  multiple,
  value,
  onChange,
  isRequired,
  isInvalid,
  ...rest
}) {
  const { onFormInputChange, formInputError, renderFormInput } = useFormInput({
    name: rest.name,
    label: rest.label,
    value,
    onChange,
    validationProps: { isRequired, isInvalid }
  });
  const handleChange = async (changeEvent) => {
    const filesPromises = Array.from(changeEvent.target.files || []).map(async (file) => {
      const fullFile = {
        name: file.name,
        type: file.type,
        size: file.size,
        base64: await readFile(file)
      };
      return fullFile;
    });
    const files = await Promise.all(filesPromises);
    onFormInputChange(files);
  };
  return renderFormInput(
    /* @__PURE__ */ React.createElement(
      MuiTextField,
      {
        ...rest,
        type: "file",
        value: void 0,
        onChange: handleChange,
        inputProps: { multiple },
        InputLabelProps: { shrink: true },
        ...formInputError && {
          error: Boolean(formInputError),
          helperText: formInputError.message || ""
        }
      }
    )
  );
}
var FormWrappedFilePicker = withComponentForm(FilePicker);
var FilePicker_default = createComponent(FormWrappedFilePicker, {
  helperText: "File Picker component.\nIt allows users to take select and read files.",
  argTypes: {
    value: {
      type: "object",
      visible: false,
      onChangeProp: "onChange"
    },
    label: {
      helperText: 'A label that describes the content of the FilePicker. e.g. "Profile Image".',
      type: "string"
    },
    multiple: {
      helperText: "Whether the FilePicker should accept multiple files.",
      type: "boolean",
      default: true
    },
    disabled: {
      helperText: "Whether the FilePicker is disabled.",
      type: "boolean"
    },
    ...FORM_INPUT_ARG_TYPES,
    sx: {
      type: "object"
    }
  }
});

export {
  FilePicker_default
};
//# sourceMappingURL=chunk-PTX72GCQ.js.map