import {
  FORM_INPUT_ARG_TYPES,
  FORM_TEXT_INPUT_ARG_TYPES,
  useFormInput,
  withComponentForm
} from "./chunk-44V7URYC.js";
import {
  SX_PROP_HELPER_TEXT
} from "./chunk-Z6S6EGNW.js";

// src/TextField.tsx
import * as React from "react";
import {
  TextField as MuiTextField
} from "@mui/material";
import { createComponent } from "@mui/toolpad-core";
function TextField({
  defaultValue,
  onChange,
  value,
  isRequired,
  minLength,
  maxLength,
  isInvalid,
  ...rest
}) {
  const { onFormInputChange, formInputError, renderFormInput } = useFormInput({
    name: rest.name,
    label: rest.label,
    value,
    onChange,
    emptyValue: "",
    defaultValue,
    validationProps: { isRequired, minLength, maxLength, isInvalid }
  });
  const handleChange = React.useCallback(
    (event) => {
      const newValue = event.target.value;
      onFormInputChange(newValue);
    },
    [onFormInputChange]
  );
  return renderFormInput(
    /* @__PURE__ */ React.createElement(
      MuiTextField,
      {
        value,
        onChange: handleChange,
        ...rest,
        ...formInputError && {
          error: Boolean(formInputError),
          helperText: formInputError.message || ""
        }
      }
    )
  );
}
var FormWrappedTextField = withComponentForm(TextField);
var TextField_default = createComponent(FormWrappedTextField, {
  helperText: "The TextField component lets you input a text value.",
  layoutDirection: "both",
  argTypes: {
    value: {
      helperText: "The value that is controlled by this text input.",
      type: "string",
      default: "",
      onChangeProp: "onChange",
      defaultValueProp: "defaultValue"
    },
    defaultValue: {
      helperText: "A default value for when the input is still empty.",
      type: "string",
      default: ""
    },
    label: {
      helperText: 'A label that describes the content of the text field. e.g. "First name".',
      type: "string"
    },
    variant: {
      helperText: "One of the available MUI TextField [variants](https://mui.com/material-ui/react-button/#basic-button). Possible values are `outlined`, `filled` or `standard`",
      type: "string",
      enum: ["outlined", "filled", "standard"],
      default: "outlined"
    },
    size: {
      helperText: "The size of the input. One of `small`, or `medium`.",
      type: "string",
      enum: ["small", "medium"],
      default: "small"
    },
    fullWidth: {
      helperText: "Whether the input should occupy all available horizontal space.",
      type: "boolean"
    },
    disabled: {
      helperText: "Whether the input is disabled.",
      type: "boolean"
    },
    ...FORM_INPUT_ARG_TYPES,
    ...FORM_TEXT_INPUT_ARG_TYPES,
    sx: {
      helperText: SX_PROP_HELPER_TEXT,
      type: "object"
    }
  }
});

export {
  TextField_default
};
//# sourceMappingURL=chunk-S3TXH4AK.js.map