"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/numberFormat.tsx
var numberFormat_exports = {};
__export(numberFormat_exports, {
  CURRENCY_CODES_LIST_HELP_URL: () => CURRENCY_CODES_LIST_HELP_URL,
  FormattedNumber: () => FormattedNumber,
  NUMBER_FORMAT_PRESETS: () => NUMBER_FORMAT_PRESETS,
  NUMBER_FORMAT_SCHEMA: () => NUMBER_FORMAT_SCHEMA,
  NumberFormatEditor: () => NumberFormatEditor,
  createFormat: () => createFormat,
  createStringFormatter: () => createStringFormatter
});
module.exports = __toCommonJS(numberFormat_exports);
var React = __toESM(require("react"), 1);
var import_material = require("@mui/material");
var ACCEPTABLE_CURRENCY_REGEX = /^[a-zA-Z]{3}$/;
var CURRENCY_CODES_LIST_HELP_URL = "https://en.wikipedia.org/wiki/ISO_4217#List_of_ISO_4217_currency_codes";
var NUMBER_FORMAT_SCHEMA = {
  anyOf: [
    {
      type: "object",
      properties: {
        kind: {
          type: "string",
          const: "preset"
        },
        preset: {
          type: "string"
        }
      },
      required: ["kind", "preset"]
    },
    {
      type: "object",
      properties: {
        kind: {
          type: "string",
          const: "currency"
        },
        currency: {
          type: "string"
        }
      },
      required: ["kind", "currency"]
    },
    {
      type: "object",
      properties: {
        kind: {
          type: "string",
          const: "custom"
        },
        custom: {
          type: "object",
          properties: {
            compactDisplay: { type: "string", enum: ["short", "long"] },
            notation: {
              type: "string",
              enum: ["standard", "scientific", "engineering", "compact"]
            },
            signDisplay: { type: "string", enum: ["auto", "never", "always", "exceptZero"] },
            unit: { type: "string" },
            unitDisplay: { type: "string", enum: ["short", "long", "narrow"] },
            currencyDisplay: { type: "string" },
            currencySign: { type: "string" }
          },
          required: []
        }
      },
      required: ["kind", "custom"]
    }
  ]
};
var NUMBER_FORMAT_PRESETS = /* @__PURE__ */ new Map([
  [
    "bytes",
    {
      label: "Bytes",
      options: {
        style: "unit",
        maximumSignificantDigits: 3,
        notation: "compact",
        unit: "byte",
        unitDisplay: "narrow"
      }
    }
  ],
  [
    "percent",
    {
      label: "Percent",
      options: {
        style: "percent"
      }
    }
  ]
]);
function createFormat(numberFormat) {
  if (!numberFormat) {
    return new Intl.NumberFormat(void 0, {});
  }
  switch (numberFormat.kind) {
    case "preset": {
      const preset = NUMBER_FORMAT_PRESETS.get(numberFormat.preset);
      return Intl.NumberFormat(void 0, preset?.options);
    }
    case "custom": {
      return Intl.NumberFormat(void 0, numberFormat.custom);
    }
    case "currency": {
      const userInput = numberFormat.currency || "USD";
      return Intl.NumberFormat(void 0, {
        style: "currency",
        currency: ACCEPTABLE_CURRENCY_REGEX.test(userInput) ? userInput : "USD"
      });
    }
    default: {
      return Intl.NumberFormat();
    }
  }
}
function createStringFormatter(numberFormat) {
  const format = createFormat(numberFormat);
  return ({ value }) => format.format(Number(value));
}
var PrettyNumberFormatRoot = (0, import_material.styled)("span")({
  "& .number-token-type-currency, & .number-token-type-percentSign": {
    // This makes the currency/percent symbol a bit smaller than the number, but only on larger font sizes
    fontSize: "max(1rem, 0.8em)"
  }
});
var DEFAULT_FORMAT = new Intl.NumberFormat();
function FormattedNumber({ children, format = DEFAULT_FORMAT }) {
  const parts = React.useMemo(() => format.formatToParts(Number(children)), [children, format]);
  return /* @__PURE__ */ React.createElement(PrettyNumberFormatRoot, null, parts.map((part, i) => /* @__PURE__ */ React.createElement("span", { key: i, className: `number-token-type-${part.type}` }, part.value)));
}
function formatNumberOptionValue(numberFormat) {
  if (!numberFormat) {
    return "plain";
  }
  switch (numberFormat.kind) {
    case "preset":
      return `preset:${numberFormat.preset}`;
    case "custom":
      return "custom";
    case "currency":
      return "currency";
    default:
      return "plain";
  }
}
function NumberFormatEditor({
  label,
  disabled,
  value,
  onChange,
  sx
}) {
  return /* @__PURE__ */ React.createElement(import_material.Stack, { sx, gap: 1 }, /* @__PURE__ */ React.createElement(
    import_material.TextField,
    {
      select: true,
      fullWidth: true,
      label: label ?? "Number format",
      value: formatNumberOptionValue(value),
      disabled,
      onChange: (event) => {
        let numberFormat;
        if (event.target.value === "currency") {
          numberFormat = {
            kind: "currency",
            currency: "USD"
          };
        } else if (event.target.value === "custom") {
          numberFormat = {
            kind: "custom",
            custom: {}
          };
        } else if (event.target.value) {
          const [prefix, id] = event.target.value.split(":");
          if (prefix === "preset") {
            numberFormat = {
              kind: "preset",
              preset: id
            };
          }
        }
        onChange(numberFormat);
      }
    },
    /* @__PURE__ */ React.createElement(import_material.MenuItem, { value: "plain" }, "Plain"),
    Array.from(NUMBER_FORMAT_PRESETS, ([type, preset]) => /* @__PURE__ */ React.createElement(import_material.MenuItem, { key: type, value: `preset:${type}` }, preset.label || type)),
    /* @__PURE__ */ React.createElement(import_material.MenuItem, { value: "currency" }, "Currency")
  ), /* @__PURE__ */ React.createElement(import_material.Box, { sx: { ml: 1, pl: 1, borderLeft: 1, borderColor: "divider" } }, value?.kind === "currency" ? /* @__PURE__ */ React.createElement(
    import_material.TextField,
    {
      fullWidth: true,
      label: "currency code",
      value: value.currency,
      disabled,
      onChange: (event) => {
        onChange({
          ...value,
          kind: "currency",
          currency: event.target.value
        });
      },
      error: !!value.currency && !ACCEPTABLE_CURRENCY_REGEX.test(value.currency),
      helperText: /* @__PURE__ */ React.createElement(React.Fragment, null, "ISO 4217 currency code. See the", " ", /* @__PURE__ */ React.createElement("a", { target: "_blank", href: CURRENCY_CODES_LIST_HELP_URL, rel: "noreferrer" }, "currency code list"), " ", "for available values.")
    }
  ) : null));
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CURRENCY_CODES_LIST_HELP_URL,
  FormattedNumber,
  NUMBER_FORMAT_PRESETS,
  NUMBER_FORMAT_SCHEMA,
  NumberFormatEditor,
  createFormat,
  createStringFormatter
});
//# sourceMappingURL=numberFormat.cjs.map