/// <reference path="../src/serverModules.d.ts" />
/// <reference types="node" />
import { IncomingMessage } from 'node:http';
import { TOOLPAD_FUNCTION } from './constants';
import { InferParameterType, PrimitiveValueType } from './types';
/**
 * The runtime configuration for a Toolpad function. Describes the parameters it accepts and their
 * corresponding types.
 */
export interface CreateFunctionConfig<C> {
    parameters: {
        [K in keyof C]: PrimitiveValueType;
    };
}
type CreateFunctionConfigParameters<C extends CreateFunctionConfig<CreateFunctionConfigParameters<C>>> = FunctionResolverParams<C>['parameters'];
export interface FunctionResolverParams<C extends CreateFunctionConfig<CreateFunctionConfigParameters<C>>> {
    parameters: {
        [K in keyof C['parameters']]: InferParameterType<C['parameters'][K]>;
    };
}
export interface FunctionResolver<C extends CreateFunctionConfig<CreateFunctionConfigParameters<C>>, R> {
    (params: FunctionResolverParams<C>): Promise<R>;
}
export interface ToolpadFunction<C extends CreateFunctionConfig<CreateFunctionConfigParameters<C>>, R> extends FunctionResolver<C, R> {
    [TOOLPAD_FUNCTION]: C;
}
/**
 * @deprecated Directly export a function instead. This will be removed in a future release.
 * See: https://mui.com/toolpad/reference/api/create-function/
 * Use this to define a function that will load the data for a Toolpad query.
 * You can define parameters for the function in the configuration object.
 * These parameters will be available in the Toolpad editor when creating a query and can be bound to page state.
 * The return value of this function will appear as state on the page and can be bound to.
 * @param resolver The function that will load the data for the query.
 * @param config The configuration for the function.
 * override: Config
 */
export declare function createFunction<C extends CreateFunctionConfig<CreateFunctionConfigParameters<C>>, R>(resolver: FunctionResolver<C, R>, config?: C): FunctionResolver<C, R> & {
    [TOOLPAD_FUNCTION]: C | {
        parameters: {};
    };
};
/**
 * @deprecated
 * createQuery is deprecated. Use createFunction instead.
 */
export declare const createQuery: typeof createFunction;
export interface ServerContext {
    cookies: Record<string, string>;
}
export declare function getContext(): ServerContext;
export declare function createServerContext(req: IncomingMessage): ServerContext;
export declare function withContext<R = void>(ctx: ServerContext, doWork: () => Promise<R>): Promise<R>;
export {};
