import {
  TOOLPAD_FUNCTION
} from "./chunk-4TRYSV22.js";

// src/server.ts
import { AsyncLocalStorage } from "async_hooks";
import * as cookie from "cookie";
function createFunction(resolver, config) {
  if (config?.parameters) {
    for (const [name, argType] of Object.entries(config.parameters)) {
      const maybeLegacyParamtype = argType;
      if (maybeLegacyParamtype.typeDef) {
        console.warn(`Detected deprecated parameter definition for "${name}".`);
        Object.assign(maybeLegacyParamtype, maybeLegacyParamtype.typeDef);
        if (!("default" in maybeLegacyParamtype)) {
          maybeLegacyParamtype.default = maybeLegacyParamtype.defaultValue;
        }
        delete maybeLegacyParamtype.defaultValue;
        delete maybeLegacyParamtype.typeDef;
      }
    }
  }
  return Object.assign(resolver, {
    [TOOLPAD_FUNCTION]: config || { parameters: {} }
  });
}
var createQuery = createFunction;
var asyncLocalStorage = new AsyncLocalStorage();
function getContext() {
  const ctx = asyncLocalStorage.getStore();
  if (!ctx) {
    throw new Error("getContext() must be called from within a Toolpad function.");
  }
  return ctx;
}
function createServerContext(req) {
  const cookies = cookie.parse(req.headers.cookie || "");
  return {
    cookies
  };
}
function withContext(ctx, doWork) {
  return asyncLocalStorage.run(ctx, doWork);
}
export {
  createFunction,
  createQuery,
  createServerContext,
  getContext,
  withContext
};
//# sourceMappingURL=server.js.map