// src/hooks/useDebouncedHandler.ts
import * as React from "react";
function clear(delayedInvocation) {
  if (delayedInvocation.current) {
    clearTimeout(delayedInvocation.current.timeout);
    delayedInvocation.current = null;
  }
}
function defer(fn, params, delay) {
  const timeout = setTimeout(() => {
    fn.current(...params);
  }, delay);
  return { startTime: Date.now(), timeout, params };
}
function useDebouncedHandler(fn, delay) {
  const fnRef = React.useRef(fn);
  React.useEffect(() => {
    fnRef.current = fn;
  }, [fn]);
  const delayedInvocation = React.useRef(null);
  React.useEffect(() => {
    if (!delayedInvocation.current) {
      return;
    }
    const { startTime, params } = delayedInvocation.current;
    const elapsed = Date.now() - startTime;
    const newDelay = Math.max(delay - elapsed, 0);
    clear(delayedInvocation);
    delayedInvocation.current = defer(fnRef, params, newDelay);
  }, [delay]);
  return React.useCallback(
    (...params) => {
      clear(delayedInvocation);
      delayedInvocation.current = defer(fnRef, params, delay);
    },
    [delay]
  );
}
export {
  useDebouncedHandler as default
};
//# sourceMappingURL=useDebouncedHandler.js.map