import {
  Emitter
} from "../chunk-WJUTEAU5.js";

// src/hooks/useStorageState.ts
import * as React from "react";
var emitter = new Emitter();
var cache = /* @__PURE__ */ new Map();
function subscribe(area, key, cb) {
  const onKeyChange = () => {
    cache.delete(key);
    cb();
  };
  const storageHandler = (event) => {
    if (event.storageArea === area && event.key === key) {
      onKeyChange();
    }
  };
  window.addEventListener("storage", storageHandler);
  emitter.on(key, onKeyChange);
  return () => {
    window.removeEventListener("storage", storageHandler);
    emitter.off(key, onKeyChange);
  };
}
function getSnapshot(area, key) {
  let value = cache.get(key) ?? null;
  if (!value) {
    const item = area.getItem(key);
    value = item;
    if (value === null) {
      cache.delete(key);
    } else {
      cache.set(key, value);
    }
  }
  return value;
}
function setValue(area, key, value) {
  if (typeof window !== "undefined") {
    if (value === null) {
      cache.delete(key);
      area.removeItem(key);
    } else {
      cache.set(key, value);
      area.setItem(key, String(value));
    }
    emitter.emit(key, null);
  }
}
function useStorageStateServer(kind, key, initializer = null) {
  const [initialValue] = React.useState(initializer);
  return [initialValue, () => {
  }];
}
function useStorageStateBrowser(kind, key, initializer = null) {
  const [initialValue] = React.useState(initializer);
  const area = kind === "session" ? window.sessionStorage : window.localStorage;
  const subscribeKey = React.useCallback((cb) => subscribe(area, key, cb), [area, key]);
  const getKeySnapshot = React.useCallback(
    () => getSnapshot(area, key) ?? initialValue,
    [area, initialValue, key]
  );
  const getKeyServerSnapshot = React.useCallback(() => initialValue, [initialValue]);
  const storedValue = React.useSyncExternalStore(
    subscribeKey,
    getKeySnapshot,
    getKeyServerSnapshot
  );
  const setStoredValue = React.useCallback(
    (value) => {
      const valueToStore = value instanceof Function ? value(storedValue) : value;
      setValue(area, key, valueToStore);
    },
    [area, key, storedValue]
  );
  return [storedValue, setStoredValue];
}
var useStorageState_default = typeof window === "undefined" ? useStorageStateServer : useStorageStateBrowser;
export {
  useStorageState_default as default
};
//# sourceMappingURL=useStorageState.js.map